@@ -1,5 +1,5 @@
 /* PSPP - a program for statistical analysis.
-   Copyright (C) 2007, 2009, 2010, 2011, 2013 Free Software Foundation, Inc.
+   Copyright (C) 2007 Free Software Foundation, Inc.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -18,10 +18,10 @@
 
    A casereader abstracts interfaces through which cases may be
    read.  A casereader may be a front-end for a system file, a
-   portable file, a dataset, or anything else on which a
-   casereader interface has been overlaid.  Casereader layering,
-   in which a casereader acts as a filter or translator on top of
-   another casereader, is also supported.
+   portable file, the active file in a data set, or anything else
+   on which a casereader interface has been overlaid.  Casereader
+   layering, in which a casereader acts as a filter or translator
+   on top of another casereader, is also supported.
 
    There is no central interface for obtaining casereaders: a
    casereader for reading a system file is obtained from the
@@ -51,39 +51,36 @@
 #ifndef DATA_CASEREADER_H
 #define DATA_CASEREADER_H 1
 
-#include "libpspp/compiler.h"
-#include "data/case.h"
-#include "data/missing-values.h"
+#include <libpspp/compiler.h>
+#include <data/case.h>
+#include <data/missing-values.h>
 
 struct dictionary;
 struct casereader;
 struct casewriter;
-struct subcase;
 
-struct ccase *casereader_read (struct casereader *);
+bool casereader_read (struct casereader *, struct ccase *);
 bool casereader_destroy (struct casereader *);
 
 struct casereader *casereader_clone (const struct casereader *);
+void casereader_split (struct casereader *,
+                       struct casereader **, struct casereader **);
 struct casereader *casereader_rename (struct casereader *);
 void casereader_swap (struct casereader *, struct casereader *);
 
-struct ccase *casereader_peek (const struct casereader *, casenumber);
-bool casereader_is_empty (const struct casereader *);
+bool casereader_peek (struct casereader *, casenumber, struct ccase *)
+     WARN_UNUSED_RESULT;
 
 bool casereader_error (const struct casereader *);
 void casereader_force_error (struct casereader *);
 const struct taint *casereader_get_taint (const struct casereader *);
 
-casenumber casereader_get_n_cases (struct casereader *);
-casenumber casereader_count_cases (const struct casereader *);
-void casereader_truncate (struct casereader *, casenumber);
-const struct caseproto *casereader_get_proto (const struct casereader *);
+casenumber casereader_get_case_cnt (struct casereader *);
+casenumber casereader_count_cases (struct casereader *);
+size_t casereader_get_value_cnt (struct casereader *);
 
-casenumber casereader_advance (struct casereader *, casenumber);
 void casereader_transfer (struct casereader *, struct casewriter *);
 
-struct casereader *casereader_create_empty (const struct caseproto *);
-
 struct casereader *
 casereader_create_filter_func (struct casereader *,
                                bool (*include) (const struct ccase *,
@@ -98,70 +95,20 @@ casereader_create_filter_weight (struct casereader *,
                                  struct casewriter *exclude);
 struct casereader *
 casereader_create_filter_missing (struct casereader *,
-                                  const struct variable *const *vars, size_t n_vars,
+                                  const struct variable **vars, size_t var_cnt,
                                   enum mv_class,
-                                  casenumber *n_missing,
                                   struct casewriter *exclude);
 
 struct casereader *
 casereader_create_counter (struct casereader *, casenumber *counter,
                            casenumber initial_value);
 
-struct casereader_translator_class
-  {
-    struct ccase *(*translate) (struct ccase *, void *aux);
-    bool (*destroy) (void *aux);
-  };
 struct casereader *
-casereader_create_translator (struct casereader *,
-                              const struct caseproto *output_proto,
-                              const struct casereader_translator_class *,
+casereader_create_translator (struct casereader *, size_t output_value_cnt,
+                              void (*translate) (struct ccase *input,
+                                                 struct ccase *output,
+                                                 void *aux),
+                              bool (*destroy) (void *aux),
                               void *aux);
-struct casereader *
-casereader_translate_stateless (struct casereader *,
-                                const struct caseproto *output_proto,
-                                const struct casereader_translator_class *,
-                                void *aux);
-
-struct casereader *casereader_project (struct casereader *,
-                                       const struct subcase *);
-struct casereader *casereader_project_1 (struct casereader *, int column);
-struct casereader *casereader_select (struct casereader *,
-                                      casenumber first, casenumber last,
-                                      casenumber by);
-
-/* A function which creates a numberic value from an existing case */
-typedef double new_value_func (const struct ccase *, casenumber, void *);
-
-struct casereader *
-casereader_create_append_numeric (struct casereader *subreader,
-                                  new_value_func func, void *aux,
-                                  void (*destroy) (void *aux));
-
-struct casereader *
-casereader_create_arithmetic_sequence (struct casereader *,
-                                       double first, double increment);
-
-enum rank_error
-  {
-    RANK_ERR_NONE = 0,
-    RANK_ERR_NEGATIVE_WEIGHT = 0x01,
-    RANK_ERR_UNSORTED = 0x02
-  };
-
-
-typedef void distinct_func (double v, casenumber n, double w, void *aux);
-
-struct casereader *
-casereader_create_append_rank (struct casereader *,
-                               const struct variable *v, const struct variable *w,
-                               enum rank_error *err,
-                               distinct_func *distinct_callback, void *aux);
-
-struct casereader *
-casereader_create_distinct (struct casereader *input,
-                            const struct variable *key,
-                            const struct variable *weight);
-
 
 #endif /* data/casereader.h */
