@@ -1,27 +1,25 @@
-/* PSPP - computes sample statistics.
+/* PSPP - a program for statistical analysis.
    Copyright (C) 1997-9, 2000, 2006 Free Software Foundation, Inc.
 
-   This program is free software; you can redistribute it and/or
-   modify it under the terms of the GNU General Public License as
-   published by the Free Software Foundation; either version 2 of the
-   License, or (at your option) any later version.
+   This program is free software: you can redistribute it and/or modify
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation, either version 3 of the License, or
+   (at your option) any later version.
 
-   This program is distributed in the hope that it will be useful, but
-   WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   General Public License for more details.
+   This program is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+   GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
-   02110-1301, USA. */
+   along with this program.  If not, see <http://www.gnu.org/licenses/>. */
 
 #include <config.h>
 #include "variable.h"
 
 #include <stdlib.h>
 
-#include "cat-routines.h"
+#include "category.h"
 #include "data-out.h"
 #include "format.h"
 #include "dictionary.h"
@@ -30,6 +28,7 @@
 #include "value-labels.h"
 #include "vardict.h"
 
+#include <libpspp/misc.h>
 #include <libpspp/alloc.h>
 #include <libpspp/assertion.h>
 #include <libpspp/compiler.h>
@@ -61,13 +60,12 @@ struct variable
     bool leave;                 /* Leave value from case to case? */
 
     /* Data for use by containing dictionary. */
-    struct vardict_info vardict;    
+    struct vardict_info vardict;
 
-    /* Short name, used only for system and portable file input
-       and output.  Upper case only. Short names are not necessarily
-       unique.  Any variable may have no short name, indicated by an
-       empty string. */
-    char short_name[SHORT_NAME_LEN + 1];
+    /* Used only for system and portable file input and output.
+       See short-names.h. */
+    char **short_names;
+    size_t short_name_cnt;
 
     /* Each command may use these fields as needed. */
     void *aux;
@@ -81,14 +79,14 @@ struct variable
 
 /* Returns true if VAR_TYPE is a valid variable type. */
 bool
-var_type_is_valid (enum var_type var_type) 
+var_type_is_valid (enum var_type var_type)
 {
   return var_type == VAR_NUMERIC || var_type == VAR_STRING;
 }
 
 /* Returns the variable type for the given width. */
 enum var_type
-var_type_from_width (int width) 
+var_type_from_width (int width)
 {
   return width != 0 ? VAR_STRING : VAR_NUMERIC;
 }
@@ -98,10 +96,10 @@ var_type_from_width (int width)
    variable is not added to a dictionary; for that, use
    dict_create_var instead. */
 struct variable *
-var_create (const char *name, int width) 
+var_create (const char *name, int width)
 {
   struct variable *v;
-  
+
   assert (width >= 0 && width <= MAX_STRING);
 
   v = xmalloc (sizeof *v);
@@ -114,20 +112,20 @@ var_create (const char *name, int width)
     {
       v->print = fmt_for_output (FMT_F, 8, 2);
       v->alignment = ALIGN_RIGHT;
-      v->display_width = 8;
       v->measure = MEASURE_SCALE;
     }
   else
     {
       v->print = fmt_for_output (FMT_A, var_get_width (v), 0);
       v->alignment = ALIGN_LEFT;
-      v->display_width = 8;
       v->measure = MEASURE_NOMINAL;
     }
+  v->display_width = var_default_display_width (width);
   v->write = v->print;
   v->val_labs = NULL;
   v->label = NULL;
-  var_clear_short_name (v);
+  v->short_names = NULL;
+  v->short_name_cnt = 0;
   v->aux = NULL;
   v->aux_dtor = NULL;
   v->obs_vals = NULL;
@@ -169,16 +167,17 @@ var_clone (const struct variable *old_var)
    V must not belong to a dictionary.  If it does, use
    dict_delete_var instead. */
 void
-var_destroy (struct variable *v) 
+var_destroy (struct variable *v)
 {
-  if (v != NULL) 
+  if (v != NULL)
     {
       assert (!var_has_vardict (v));
       cat_stored_values_destroy (v->obs_vals);
+      var_clear_short_names (v);
       var_clear_aux (v);
       val_labs_destroy (v->val_labs);
       var_clear_label (v);
-      free (v); 
+      free (v);
     }
 }
 
@@ -186,7 +185,7 @@ var_destroy (struct variable *v)
 
 /* Return variable V's name. */
 const char *
-var_get_name (const struct variable *v) 
+var_get_name (const struct variable *v)
 {
   return v->name;
 }
@@ -195,7 +194,7 @@ var_get_name (const struct variable *v)
    Do not use this function for a variable in a dictionary.  Use
    dict_rename_var instead. */
 void
-var_set_name (struct variable *v, const char *name) 
+var_set_name (struct variable *v, const char *name)
 {
   assert (v->vardict.dict_index == -1);
   assert (var_is_plausible_name (name, false));
@@ -208,20 +207,20 @@ var_set_name (struct variable *v, const char *name)
    false otherwise.  If ISSUE_ERROR is true, issues an
    explanatory error message on failure. */
 bool
-var_is_valid_name (const char *name, bool issue_error) 
+var_is_valid_name (const char *name, bool issue_error)
 {
   bool plausible;
   size_t length, i;
-  
+
   assert (name != NULL);
 
-  /* Note that strlen returns number of BYTES, not the number of 
+  /* Note that strlen returns number of BYTES, not the number of
      CHARACTERS */
   length = strlen (name);
 
   plausible = var_is_plausible_name(name, issue_error);
 
-  if ( ! plausible ) 
+  if ( ! plausible )
     return false;
 
 
@@ -237,7 +236,7 @@ var_is_valid_name (const char *name, bool issue_error)
 
   for (i = 0; i < length; i++)
     {
-    if (!lex_is_idn (name[i])) 
+    if (!lex_is_idn (name[i]))
       {
         if (issue_error)
           msg (SE, _("Character `%c' (in %s) may not appear in "
@@ -252,26 +251,26 @@ var_is_valid_name (const char *name, bool issue_error)
 
 /* Returns true if NAME is an plausible name for a variable,
    false otherwise.  If ISSUE_ERROR is true, issues an
-   explanatory error message on failure. 
+   explanatory error message on failure.
    This function makes no use of LC_CTYPE.
 */
 bool
-var_is_plausible_name (const char *name, bool issue_error) 
+var_is_plausible_name (const char *name, bool issue_error)
 {
   size_t length;
-  
+
   assert (name != NULL);
 
-  /* Note that strlen returns number of BYTES, not the number of 
+  /* Note that strlen returns number of BYTES, not the number of
      CHARACTERS */
   length = strlen (name);
-  if (length < 1) 
+  if (length < 1)
     {
       if (issue_error)
         msg (SE, _("Variable name cannot be empty string."));
       return false;
     }
-  else if (length > LONG_NAME_LEN) 
+  else if (length > LONG_NAME_LEN)
     {
       if (issue_error)
         msg (SE, _("Variable name %s exceeds %d-character limit."),
@@ -279,7 +278,7 @@ var_is_plausible_name (const char *name, bool issue_error)
       return false;
     }
 
-  if (lex_id_to_token (ss_cstr (name)) != T_ID) 
+  if (lex_id_to_token (ss_cstr (name)) != T_ID)
     {
       if (issue_error)
         msg (SE, _("`%s' may not be used as a variable name because it "
@@ -293,7 +292,7 @@ var_is_plausible_name (const char *name, bool issue_error)
 /* A hsh_compare_func that orders variables A and B by their
    names. */
 int
-compare_vars_by_name (const void *a_, const void *b_, const void *aux UNUSED) 
+compare_vars_by_name (const void *a_, const void *b_, const void *aux UNUSED)
 {
   const struct variable *a = a_;
   const struct variable *b = b_;
@@ -303,7 +302,7 @@ compare_vars_by_name (const void *a_, const void *b_, const void *aux UNUSED)
 
 /* A hsh_hash_func that hashes variable V based on its name. */
 unsigned
-hash_var_by_name (const void *v_, const void *aux UNUSED) 
+hash_var_by_name (const void *v_, const void *aux UNUSED)
 {
   const struct variable *v = v_;
 
@@ -314,7 +313,7 @@ hash_var_by_name (const void *v_, const void *aux UNUSED)
    by their names. */
 int
 compare_var_ptrs_by_name (const void *a_, const void *b_,
-                          const void *aux UNUSED) 
+                          const void *aux UNUSED)
 {
   struct variable *const *a = a_;
   struct variable *const *b = b_;
@@ -325,7 +324,7 @@ compare_var_ptrs_by_name (const void *a_, const void *b_,
 /* A hsh_hash_func that hashes pointer to variable V based on its
    name. */
 unsigned
-hash_var_ptr_by_name (const void *v_, const void *aux UNUSED) 
+hash_var_ptr_by_name (const void *v_, const void *aux UNUSED)
 {
   struct variable *const *v = v_;
 
@@ -334,14 +333,14 @@ hash_var_ptr_by_name (const void *v_, const void *aux UNUSED)
 
 /* Returns the type of variable V. */
 enum var_type
-var_get_type (const struct variable *v) 
+var_get_type (const struct variable *v)
 {
   return var_type_from_width (v->width);
 }
 
 /* Returns the width of variable V. */
 int
-var_get_width (const struct variable *v) 
+var_get_width (const struct variable *v)
 {
   return v->width;
 }
@@ -350,7 +349,7 @@ var_get_width (const struct variable *v)
 void
 var_set_width (struct variable *v, int new_width)
 {
-  enum var_type new_type = var_type_from_width (new_width);
+  const int old_width = v->width;
 
   if (mv_is_resizable (&v->miss, new_width))
     mv_resize (&v->miss, new_width);
@@ -368,27 +367,25 @@ var_set_width (struct variable *v, int new_width)
         }
     }
 
-  if (var_get_type (v) != new_type)
-    {
-      v->print = (new_type == VAR_NUMERIC
-                  ? fmt_for_output (FMT_F, 8, 2)
-                  : fmt_for_output (FMT_A, new_width, 0));
-      v->write = v->print;
-    }
-  else if (new_type == VAR_STRING) 
-    {
-      v->print.w = v->print.type == FMT_AHEX ? new_width * 2 : new_width;
-      v->write.w = v->write.type == FMT_AHEX ? new_width * 2 : new_width;
-    }
+  fmt_resize (&v->print, new_width);
+  fmt_resize (&v->write, new_width);
 
   v->width = new_width;
 
+  {
+    const int old_val_count = value_cnt_from_width (old_width);
+    const int new_val_count = value_cnt_from_width (new_width);
+
+    if ( old_val_count != new_val_count)
+	 dict_var_resized (v, new_val_count - old_val_count);
+  }
+
   dict_var_changed (v);
 }
 
 /* Returns true if variable V is numeric, false otherwise. */
 bool
-var_is_numeric (const struct variable *v) 
+var_is_numeric (const struct variable *v)
 {
   return var_get_type (v) == VAR_NUMERIC;
 }
@@ -396,7 +393,7 @@ var_is_numeric (const struct variable *v)
 /* Returns true if variable V is a string variable, false
    otherwise. */
 bool
-var_is_alpha (const struct variable *v) 
+var_is_alpha (const struct variable *v)
 {
   return var_get_type (v) == VAR_STRING;
 }
@@ -404,7 +401,7 @@ var_is_alpha (const struct variable *v)
 /* Returns true if variable V is a short string variable, false
    otherwise. */
 bool
-var_is_short_string (const struct variable *v) 
+var_is_short_string (const struct variable *v)
 {
   return v->width > 0 && v->width <= MAX_SHORT_STRING;
 }
@@ -412,7 +409,7 @@ var_is_short_string (const struct variable *v)
 /* Returns true if variable V is a long string variable, false
    otherwise. */
 bool
-var_is_long_string (const struct variable *v) 
+var_is_long_string (const struct variable *v)
 {
   return v->width > MAX_SHORT_STRING;
 }
@@ -420,14 +417,14 @@ var_is_long_string (const struct variable *v)
 /* Returns the number of "union value"s need to store a value of
    variable V. */
 size_t
-var_get_value_cnt (const struct variable *v) 
+var_get_value_cnt (const struct variable *v)
 {
-  return v->width == 0 ? 1 : DIV_RND_UP (v->width, MAX_SHORT_STRING);
+  return value_cnt_from_width (v->width);
 }
 
 /* Returns variable V's missing values. */
 const struct missing_values *
-var_get_missing_values (const struct variable *v) 
+var_get_missing_values (const struct variable *v)
 {
   return &v->miss;
 }
@@ -439,7 +436,7 @@ var_get_missing_values (const struct variable *v)
 void
 var_set_missing_values (struct variable *v, const struct missing_values *miss)
 {
-  if (miss != NULL) 
+  if (miss != NULL)
     {
       assert (mv_is_resizable (miss, v->width));
       mv_copy (&v->miss, miss);
@@ -453,7 +450,7 @@ var_set_missing_values (struct variable *v, const struct missing_values *miss)
 
 /* Sets variable V to have no user-missing values. */
 void
-var_clear_missing_values (struct variable *v) 
+var_clear_missing_values (struct variable *v)
 {
   var_set_missing_values (v, NULL);
 }
@@ -461,7 +458,7 @@ var_clear_missing_values (struct variable *v)
 /* Returns true if V has any user-missing values,
    false otherwise. */
 bool
-var_has_missing_values (const struct variable *v) 
+var_has_missing_values (const struct variable *v)
 {
   return !mv_is_empty (&v->miss);
 }
@@ -470,7 +467,7 @@ var_has_missing_values (const struct variable *v)
    in V, false otherwise. */
 bool
 var_is_value_missing (const struct variable *v, const union value *value,
-                      enum mv_class class) 
+                      enum mv_class class)
 {
   return mv_is_value_missing (&v->miss, value, class);
 }
@@ -479,7 +476,7 @@ var_is_value_missing (const struct variable *v, const union value *value,
    V, false otherwise.
    V must be a numeric variable. */
 bool
-var_is_num_missing (const struct variable *v, double d, enum mv_class class) 
+var_is_num_missing (const struct variable *v, double d, enum mv_class class)
 {
   return mv_is_num_missing (&v->miss, d, class);
 }
@@ -489,7 +486,7 @@ var_is_num_missing (const struct variable *v, double d, enum mv_class class)
    V must be a string variable. */
 bool
 var_is_str_missing (const struct variable *v, const char s[],
-                    enum mv_class class) 
+                    enum mv_class class)
 {
   return mv_is_str_missing (&v->miss, s, class);
 }
@@ -497,14 +494,14 @@ var_is_str_missing (const struct variable *v, const char s[],
 /* Returns variable V's value labels,
    possibly a null pointer if it has none. */
 const struct val_labs *
-var_get_value_labels (const struct variable *v) 
+var_get_value_labels (const struct variable *v)
 {
   return v->val_labs;
 }
 
 /* Returns true if variable V has at least one value label. */
 bool
-var_has_value_labels (const struct variable *v) 
+var_has_value_labels (const struct variable *v)
 {
   return val_labs_count (v->val_labs) > 0;
 }
@@ -514,7 +511,7 @@ var_has_value_labels (const struct variable *v)
    changed to V's width.
    If VLS is null, then V's value labels, if any, are removed. */
 void
-var_set_value_labels (struct variable *v, const struct val_labs *vls) 
+var_set_value_labels (struct variable *v, const struct val_labs *vls)
 {
   val_labs_destroy (v->val_labs);
   v->val_labs = NULL;
@@ -531,7 +528,7 @@ var_set_value_labels (struct variable *v, const struct val_labs *vls)
 /* Makes sure that V has a set of value labels,
    by assigning one to it if necessary. */
 static void
-alloc_value_labels (struct variable *v) 
+alloc_value_labels (struct variable *v)
 {
   assert (!var_is_long_string (v));
   if (v->val_labs == NULL)
@@ -544,7 +541,7 @@ alloc_value_labels (struct variable *v)
    V must not be a long string variable. */
 bool
 var_add_value_label (struct variable *v,
-                     const union value *value, const char *label) 
+                     const union value *value, const char *label)
 {
   alloc_value_labels (v);
   return val_labs_add (v->val_labs, *value, label);
@@ -563,7 +560,7 @@ var_replace_value_label (struct variable *v,
 
 /* Removes V's value labels, if any. */
 void
-var_clear_value_labels (struct variable *v) 
+var_clear_value_labels (struct variable *v)
 {
   var_set_value_labels (v, NULL);
 }
@@ -571,7 +568,7 @@ var_clear_value_labels (struct variable *v)
 /* Returns the label associated with VALUE for variable V,
    or a null pointer if none. */
 const char *
-var_lookup_value_label (const struct variable *v, const union value *value) 
+var_lookup_value_label (const struct variable *v, const union value *value)
 {
   return val_labs_find (v->val_labs, *value);
 }
@@ -583,7 +580,7 @@ const char *
 var_get_value_name (const struct variable *v, const union value *value)
 {
   const char *name = var_lookup_value_label (v, value);
-  if (name == NULL) 
+  if (name == NULL)
     {
       static char buf[MAX_STRING + 1];
       data_out (value, &v->print, buf);
@@ -597,7 +594,7 @@ var_get_value_name (const struct variable *v, const union value *value)
 
 /* Returns V's print format specification. */
 const struct fmt_spec *
-var_get_print_format (const struct variable *v) 
+var_get_print_format (const struct variable *v)
 {
   return &v->print;
 }
@@ -606,7 +603,7 @@ var_get_print_format (const struct variable *v)
    valid format specification for outputting a variable of V's
    width. */
 void
-var_set_print_format (struct variable *v, const struct fmt_spec *print) 
+var_set_print_format (struct variable *v, const struct fmt_spec *print)
 {
   assert (fmt_check_width_compat (print, v->width));
   v->print = *print;
@@ -615,7 +612,7 @@ var_set_print_format (struct variable *v, const struct fmt_spec *print)
 
 /* Returns V's write format specification. */
 const struct fmt_spec *
-var_get_write_format (const struct variable *v) 
+var_get_write_format (const struct variable *v)
 {
   return &v->write;
 }
@@ -624,7 +621,7 @@ var_get_write_format (const struct variable *v)
    valid format specification for outputting a variable of V's
    width. */
 void
-var_set_write_format (struct variable *v, const struct fmt_spec *write) 
+var_set_write_format (struct variable *v, const struct fmt_spec *write)
 {
   assert (fmt_check_width_compat (write, v->width));
   v->write = *write;
@@ -635,7 +632,7 @@ var_set_write_format (struct variable *v, const struct fmt_spec *write)
    which must be a valid format specification for outputting a
    variable of V's width. */
 void
-var_set_both_formats (struct variable *v, const struct fmt_spec *format) 
+var_set_both_formats (struct variable *v, const struct fmt_spec *format)
 {
   var_set_print_format (v, format);
   var_set_write_format (v, format);
@@ -652,7 +649,7 @@ var_to_string (const struct variable *v)
 
 /* Returns V's variable label, or a null pointer if it has none. */
 const char *
-var_get_label (const struct variable *v) 
+var_get_label (const struct variable *v)
 {
   return v->label;
 }
@@ -663,17 +660,17 @@ var_get_label (const struct variable *v)
    (after stripping white space), then V's variable label (if
    any) is removed. */
 void
-var_set_label (struct variable *v, const char *label) 
+var_set_label (struct variable *v, const char *label)
 {
   free (v->label);
   v->label = NULL;
 
-  if (label != NULL) 
+  if (label != NULL)
     {
       struct substring s = ss_cstr (label);
       ss_trim (&s, ss_cstr (CC_SPACES));
       ss_truncate (&s, 255);
-      if (!ss_is_empty (s)) 
+      if (!ss_is_empty (s))
         v->label = ss_xstrdup (s);
       dict_var_changed (v);
     }
@@ -681,7 +678,7 @@ var_set_label (struct variable *v, const char *label)
 
 /* Removes any variable label from V. */
 void
-var_clear_label (struct variable *v) 
+var_clear_label (struct variable *v)
 {
   var_set_label (v, NULL);
 }
@@ -689,7 +686,7 @@ var_clear_label (struct variable *v)
 /* Returns true if V has a variable V,
    false otherwise. */
 bool
-var_has_label (const struct variable *v) 
+var_has_label (const struct variable *v)
 {
   return v->label != NULL;
 }
@@ -704,14 +701,14 @@ measure_is_valid (enum measure m)
 
 /* Returns V's measurement level. */
 enum measure
-var_get_measure (const struct variable *v) 
+var_get_measure (const struct variable *v)
 {
   return v->measure;
 }
 
 /* Sets V's measurement level to MEASURE. */
 void
-var_set_measure (struct variable *v, enum measure measure) 
+var_set_measure (struct variable *v, enum measure measure)
 {
   assert (measure_is_valid (measure));
   v->measure = measure;
@@ -720,21 +717,27 @@ var_set_measure (struct variable *v, enum measure measure)
 
 /* Returns V's display width, which applies only to GUIs. */
 int
-var_get_display_width (const struct variable *v) 
+var_get_display_width (const struct variable *v)
 {
   return v->display_width;
 }
 
-
-
-
 /* Sets V's display width to DISPLAY_WIDTH. */
 void
-var_set_display_width (struct variable *v, int display_width) 
+var_set_display_width (struct variable *v, int display_width)
 {
   v->display_width = display_width;
   dict_var_changed (v);
 }
+
+/* Returns the default display width for a variable of the given
+   WIDTH, as set by var_create.  The return value can be used to
+   reset a variable's display width to the default. */
+int
+var_default_display_width (int width)
+{
+  return width == 0 ? 8 : MIN (width, 32);
+}
 
 /* Returns true if A is a valid alignment,
    false otherwise. */
@@ -746,14 +749,14 @@ alignment_is_valid (enum alignment a)
 
 /* Returns V's display alignment, which applies only to GUIs. */
 enum alignment
-var_get_alignment (const struct variable *v) 
+var_get_alignment (const struct variable *v)
 {
   return v->alignment;
 }
 
 /* Sets V's display alignment to ALIGNMENT. */
 void
-var_set_alignment (struct variable *v, enum alignment alignment) 
+var_set_alignment (struct variable *v, enum alignment alignment)
 {
   assert (alignment_is_valid (alignment));
   v->alignment = alignment;
@@ -766,14 +769,14 @@ var_set_alignment (struct variable *v, enum alignment alignment)
 /* Returns true if variable V's value should be left from case to
    case, instead of being reset to 0, system-missing, or blanks. */
 bool
-var_get_leave (const struct variable *v) 
+var_get_leave (const struct variable *v)
 {
   return v->leave;
 }
 
 /* Sets V's leave setting to LEAVE. */
 void
-var_set_leave (struct variable *v, bool leave) 
+var_set_leave (struct variable *v, bool leave)
 {
   assert (leave || !var_must_leave (v));
   v->leave = leave;
@@ -783,52 +786,85 @@ var_set_leave (struct variable *v, bool leave)
 /* Returns true if V must be left from case to case,
    false if it can be set either way. */
 bool
-var_must_leave (const struct variable *v) 
+var_must_leave (const struct variable *v)
 {
   return dict_class_from_id (v->name) == DC_SCRATCH;
 }
 
-/* Returns V's short name, if it has one, or a null pointer
-   otherwise.
+/* Returns the number of short names stored in VAR.
 
    Short names are used only for system and portable file input
    and output.  They are upper-case only, not necessarily unique,
    and limited to SHORT_NAME_LEN characters (plus a null
-   terminator).  Any variable may have no short name, indicated
-   by returning a null pointer. */
-const char *
-var_get_short_name (const struct variable *v) 
+   terminator).  Ordinarily a variable has at most one short
+   name, but very long string variables (longer than 255 bytes)
+   may have more.  A variable might not have any short name at
+   all if it hasn't been saved to or read from a system or
+   portable file. */
+size_t
+var_get_short_name_cnt (const struct variable *var) 
 {
-  return v->short_name[0] != '\0' ? v->short_name : NULL;
+  return var->short_name_cnt;
 }
 
-/* Sets V's short_name to SHORT_NAME, truncating it to
-   SHORT_NAME_LEN characters and converting it to uppercase in
-   the process.  Specifying a null pointer for SHORT_NAME clears
-   the variable's short name. */
-void
-var_set_short_name (struct variable *v, const char *short_name) 
+/* Returns VAR's short name with the given IDX, if it has one
+   with that index, or a null pointer otherwise.  Short names may
+   be sparse: even if IDX is less than the number of short names
+   in VAR, this function may return a null pointer. */
+const char *
+var_get_short_name (const struct variable *var, size_t idx)
 {
-  assert (v != NULL);
+  return idx < var->short_name_cnt ? var->short_names[idx] : NULL;
+}
+
+/* Sets VAR's short name with the given IDX to SHORT_NAME,
+   truncating it to SHORT_NAME_LEN characters and converting it
+   to uppercase in the process.  Specifying a null pointer for
+   SHORT_NAME clears the specified short name. */
+void
+var_set_short_name (struct variable *var, size_t idx, const char *short_name)
+{
+  assert (var != NULL);
   assert (short_name == NULL || var_is_plausible_name (short_name, false));
 
+  /* Clear old short name numbered IDX, if any. */
+  if (idx < var->short_name_cnt) 
+    {
+      free (var->short_names[idx]);
+      var->short_names[idx] = NULL; 
+    }
+
+  /* Install new short name for IDX. */
   if (short_name != NULL) 
     {
-      str_copy_trunc (v->short_name, sizeof v->short_name, short_name);
-      str_uppercase (v->short_name); 
+      if (idx >= var->short_name_cnt)
+        {
+          size_t old_cnt = var->short_name_cnt;
+          size_t i;
+          var->short_name_cnt = MAX (idx * 2, 1);
+          var->short_names = xnrealloc (var->short_names, var->short_name_cnt,
+                                        sizeof *var->short_names);
+          for (i = old_cnt; i < var->short_name_cnt; i++)
+            var->short_names[i] = NULL;
+        }
+      var->short_names[idx] = xstrndup (short_name, MAX_SHORT_STRING);
+      str_uppercase (var->short_names[idx]);
     }
-  else
-    v->short_name[0] = '\0';
-  dict_var_changed (v);
+
+  dict_var_changed (var);
 }
 
-/* Clears V's short name. */
+/* Clears V's short names. */
 void
-var_clear_short_name (struct variable *v) 
+var_clear_short_names (struct variable *v)
 {
-  assert (v != NULL);
+  size_t i;
 
-  v->short_name[0] = '\0';
+  for (i = 0; i < v->short_name_cnt; i++)
+    free (v->short_names[i]);
+  free (v->short_names);
+  v->short_names = NULL;
+  v->short_name_cnt = 0;
 }
 
 /* Relationship with dictionary. */
@@ -837,7 +873,7 @@ var_clear_short_name (struct variable *v)
    for which "dict_get_var (dict, index)" will return V.
    V must be in a dictionary. */
 size_t
-var_get_dict_index (const struct variable *v) 
+var_get_dict_index (const struct variable *v)
 {
   assert (v->vardict.dict_index != -1);
   return v->vardict.dict_index;
@@ -848,7 +884,7 @@ var_get_dict_index (const struct variable *v)
    index)" will return the data for V in that case.
    V must be in a dictionary. */
 size_t
-var_get_case_index (const struct variable *v) 
+var_get_case_index (const struct variable *v)
 {
   assert (v->vardict.case_index != -1);
   return v->vardict.case_index;
@@ -857,7 +893,7 @@ var_get_case_index (const struct variable *v)
 /* Returns V's auxiliary data, or a null pointer if none has been
    attached. */
 void *
-var_get_aux (const struct variable *v) 
+var_get_aux (const struct variable *v)
 {
   return v->aux;
 }
@@ -868,7 +904,7 @@ var_get_aux (const struct variable *v)
    may be appropriate for use as AUX_DTOR.) */
 void *
 var_attach_aux (const struct variable *v_,
-                void *aux, void (*aux_dtor) (struct variable *)) 
+                void *aux, void (*aux_dtor) (struct variable *))
 {
   struct variable *v = (struct variable *) v_ ; /* cast away const  */
   assert (v->aux == NULL);
@@ -881,7 +917,7 @@ var_attach_aux (const struct variable *v_,
 /* Remove auxiliary data, if any, from V, and return it, without
    calling any associated destructor. */
 void *
-var_detach_aux (struct variable *v) 
+var_detach_aux (struct variable *v)
 {
   void *aux = v->aux;
   assert (aux != NULL);
@@ -892,10 +928,10 @@ var_detach_aux (struct variable *v)
 /* Clears auxiliary data, if any, from V, and calls any
    associated destructor. */
 void
-var_clear_aux (struct variable *v) 
+var_clear_aux (struct variable *v)
 {
   assert (v != NULL);
-  if (v->aux != NULL) 
+  if (v->aux != NULL)
     {
       if (v->aux_dtor != NULL)
         v->aux_dtor (v);
@@ -907,7 +943,7 @@ var_clear_aux (struct variable *v)
    destructor (passed as AUX_DTOR to var_attach_aux()) for the
    case where the auxiliary data should be passed to free(). */
 void
-var_dtor_free (struct variable *v) 
+var_dtor_free (struct variable *v)
 {
   free (v->aux);
 }
@@ -917,7 +953,7 @@ var_dtor_free (struct variable *v)
 /* Returns V's observed categorical values,
    which V must have. */
 struct cat_vals *
-var_get_obs_vals (const struct variable *v) 
+var_get_obs_vals (const struct variable *v)
 {
   assert (v->obs_vals != NULL);
   return v->obs_vals;
@@ -925,9 +961,9 @@ var_get_obs_vals (const struct variable *v)
 
 /* Sets V's observed categorical values to CAT_VALS. */
 void
-var_set_obs_vals (const struct variable *v_, struct cat_vals *cat_vals) 
+var_set_obs_vals (const struct variable *v_, struct cat_vals *cat_vals)
 {
-  struct variable *v = (struct variable *) v_ ; /* cast away const */ 
+  struct variable *v = (struct variable *) v_ ; /* cast away const */
   cat_stored_values_destroy (v->obs_vals);
   v->obs_vals = cat_vals;
 }
@@ -935,7 +971,7 @@ var_set_obs_vals (const struct variable *v_, struct cat_vals *cat_vals)
 /* Returns true if V has observed categorical values,
    false otherwise. */
 bool
-var_has_obs_vals (const struct variable *v) 
+var_has_obs_vals (const struct variable *v)
 {
   return v->obs_vals != NULL;
 }
@@ -943,9 +979,9 @@ var_has_obs_vals (const struct variable *v)
 /* Returns the dictionary class corresponding to a variable named
    NAME. */
 enum dict_class
-dict_class_from_id (const char *name) 
+dict_class_from_id (const char *name)
 {
-  switch (name[0]) 
+  switch (name[0])
     {
     default:
       return DC_ORDINARY;
@@ -958,9 +994,9 @@ dict_class_from_id (const char *name)
 
 /* Returns the name of dictionary class DICT_CLASS. */
 const char *
-dict_class_to_name (enum dict_class dict_class) 
+dict_class_to_name (enum dict_class dict_class)
 {
-  switch (dict_class) 
+  switch (dict_class)
     {
     case DC_ORDINARY:
       return _("ordinary");
@@ -975,7 +1011,7 @@ dict_class_to_name (enum dict_class dict_class)
 
 /* Returns V's vardict structure. */
 const struct vardict_info *
-var_get_vardict (const struct variable *v) 
+var_get_vardict (const struct variable *v)
 {
   assert (var_has_vardict (v));
   return &v->vardict;
@@ -983,7 +1019,7 @@ var_get_vardict (const struct variable *v)
 
 /* Sets V's vardict data to VARDICT. */
 void
-var_set_vardict (struct variable *v, const struct vardict_info *vardict) 
+var_set_vardict (struct variable *v, const struct vardict_info *vardict)
 {
   assert (vardict->dict_index >= 0);
   assert (vardict->case_index >= 0);
@@ -992,14 +1028,14 @@ var_set_vardict (struct variable *v, const struct vardict_info *vardict)
 
 /* Returns true if V has vardict data. */
 bool
-var_has_vardict (const struct variable *v) 
+var_has_vardict (const struct variable *v)
 {
   return v->vardict.dict_index != -1;
 }
 
 /* Clears V's vardict data. */
 void
-var_clear_vardict (struct variable *v) 
+var_clear_vardict (struct variable *v)
 {
   v->vardict.dict_index = v->vardict.case_index = -1;
 }
