@@ -1,26 +1,25 @@
-/* PSPP - computes sample statistics.
+/* PSPP - a program for statistical analysis.
    Copyright (C) 1997-9, 2000 Free Software Foundation, Inc.
 
-   This program is free software; you can redistribute it and/or
-   modify it under the terms of the GNU General Public License as
-   published by the Free Software Foundation; either version 2 of the
-   License, or (at your option) any later version.
+   This program is free software: you can redistribute it and/or modify
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation, either version 3 of the License, or
+   (at your option) any later version.
 
-   This program is distributed in the hope that it will be useful, but
-   WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   General Public License for more details.
+   This program is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+   GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
-   02110-1301, USA. */
+   along with this program.  If not, see <http://www.gnu.org/licenses/>. */
 
 #include <config.h>
 
 #include <stdlib.h>
 
 #include "control-stack.h"
+#include <data/case.h>
 #include <data/procedure.h>
 #include <data/transformations.h>
 #include <data/value.h>
@@ -44,7 +43,7 @@
 
    So, the following code:
 
-       DO IF a.             
+       DO IF a.
        ...block 1...
        ELSE IF b.
        ...block 2...
@@ -65,7 +64,7 @@
 */
 
 /* A conditional clause. */
-struct clause 
+struct clause
   {
     struct expression *condition; /* Test expression; NULL for ELSE clause. */
     int target_index;           /* Transformation to jump to if true. */
@@ -83,8 +82,7 @@ struct do_if_trns
 static const struct ctl_class do_if_class;
 
 static int parse_clause (struct lexer *, struct do_if_trns *, struct dataset *ds);
-static void add_clause (struct do_if_trns *,
-                        struct expression *condition, int target_index);
+static void add_clause (struct do_if_trns *, struct expression *condition);
 static void add_else (struct do_if_trns *);
 
 static bool has_else (struct do_if_trns *);
@@ -154,8 +152,8 @@ static void
 close_do_if (void *do_if_)
 {
   struct do_if_trns *do_if = do_if_;
-  
-  if (!has_else (do_if)) 
+
+  if (!has_else (do_if))
     add_else (do_if);
   do_if->past_END_IF_index = next_transformation (do_if->ds);
 }
@@ -163,16 +161,16 @@ close_do_if (void *do_if_)
 /* Adds an ELSE clause to DO_IF pointing to the next
    transformation. */
 static void
-add_else (struct do_if_trns *do_if) 
+add_else (struct do_if_trns *do_if)
 {
   assert (!has_else (do_if));
-  add_clause (do_if, NULL, next_transformation (do_if->ds));
+  add_clause (do_if, NULL);
 }
 
 /* Returns true if DO_IF does not yet have an ELSE clause.
    Reports an error and returns false if it does already. */
 static bool
-must_not_have_else (struct do_if_trns *do_if) 
+must_not_have_else (struct do_if_trns *do_if)
 {
   if (has_else (do_if))
     {
@@ -186,7 +184,7 @@ must_not_have_else (struct do_if_trns *do_if)
 /* Returns true if DO_IF already has an ELSE clause,
    false otherwise. */
 static bool
-has_else (struct do_if_trns *do_if) 
+has_else (struct do_if_trns *do_if)
 {
   return (do_if->clause_cnt != 0
           && do_if->clauses[do_if->clause_cnt - 1].condition == NULL);
@@ -204,16 +202,16 @@ parse_clause (struct lexer *lexer, struct do_if_trns *do_if, struct dataset *ds)
   if (condition == NULL)
     return CMD_CASCADING_FAILURE;
 
-  add_clause (do_if, condition, next_transformation (ds));
+  add_clause (do_if, condition);
 
   return lex_end_of_command (lexer);
 }
 
 /* Adds a clause to DO_IF that tests for the given CONDITION and,
-   if true, jumps to TARGET_INDEX. */
+   if true, jumps to the set of transformations produced by
+   following commands. */
 static void
-add_clause (struct do_if_trns *do_if,
-            struct expression *condition, int target_index) 
+add_clause (struct do_if_trns *do_if, struct expression *condition)
 {
   struct clause *clause;
 
@@ -224,13 +222,13 @@ add_clause (struct do_if_trns *do_if,
                               do_if->clause_cnt + 1, sizeof *do_if->clauses);
   clause = &do_if->clauses[do_if->clause_cnt++];
   clause->condition = condition;
-  clause->target_index = target_index;
+  clause->target_index = next_transformation (do_if->ds);
 }
 
 /* Finalizes DO IF by clearing the control stack, thus ensuring
-   that all open DO IFs are closed. */ 
+   that all open DO IFs are closed. */
 static void
-do_if_finalize_func (void *do_if_ UNUSED) 
+do_if_finalize_func (void *do_if_ UNUSED)
 {
   /* This will be called multiple times if multiple DO IFs were
      executed, which is slightly unclean, but at least it's
@@ -241,14 +239,14 @@ do_if_finalize_func (void *do_if_ UNUSED)
 /* DO IF transformation procedure.
    Checks each clause and jumps to the appropriate
    transformation. */
-static int 
+static int
 do_if_trns_proc (void *do_if_, struct ccase *c, casenumber case_num UNUSED)
 {
   struct do_if_trns *do_if = do_if_;
   struct clause *clause;
 
   for (clause = do_if->clauses; clause < do_if->clauses + do_if->clause_cnt;
-       clause++) 
+       clause++)
     {
       if (clause->condition != NULL)
         {
@@ -258,7 +256,7 @@ do_if_trns_proc (void *do_if_, struct ccase *c, casenumber case_num UNUSED)
           else if (boolean == SYSMIS)
             return do_if->past_END_IF_index;
         }
-      else 
+      else
         return clause->target_index;
     }
   return do_if->past_END_IF_index;
@@ -280,7 +278,7 @@ do_if_trns_free (void *do_if_)
 }
 
 /* Breaks out of a DO IF construct. */
-static int 
+static int
 break_trns_proc (void *do_if_, struct ccase *c UNUSED, casenumber case_num UNUSED)
 {
   struct do_if_trns *do_if = do_if_;
@@ -289,7 +287,7 @@ break_trns_proc (void *do_if_, struct ccase *c UNUSED, casenumber case_num UNUSE
 }
 
 /* DO IF control structure class definition. */
-static const struct ctl_class do_if_class = 
+static const struct ctl_class do_if_class =
   {
     "DO IF",
     "END IF",
