@@ -1,20 +1,18 @@
-/* PSPP - computes sample statistics.
+/* PSPP - a program for statistical analysis.
    Copyright (C) 1997-9, 2000 Free Software Foundation, Inc.
 
-   This program is free software; you can redistribute it and/or
-   modify it under the terms of the GNU General Public License as
-   published by the Free Software Foundation; either version 2 of the
-   License, or (at your option) any later version.
+   This program is free software: you can redistribute it and/or modify
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation, either version 3 of the License, or
+   (at your option) any later version.
 
-   This program is distributed in the hope that it will be useful, but
-   WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   General Public License for more details.
+   This program is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+   GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
-   02110-1301, USA. */
+   along with this program.  If not, see <http://www.gnu.org/licenses/>. */
 
 #include <config.h>
 
@@ -40,7 +38,6 @@
 #include "gettext.h"
 #define _(msgid) gettext (msgid)
 
-/* FIXME: should change weighting variable, etc. */
 /* These control the ordering produced by
    compare_variables_given_ordering(). */
 struct ordering
@@ -193,7 +190,7 @@ cmd_modify_vars (struct lexer *lexer, struct dataset *ds)
 		  msg (SE, _("`(' expected on RENAME subcommand."));
 		  goto done;
 		}
-	      if (!parse_variables (lexer, dataset_dict (ds), 
+	      if (!parse_variables (lexer, dataset_dict (ds),
 				    &vm.rename_vars, &vm.rename_cnt,
 				    PV_APPEND | PV_NO_DUPLICATE))
 		goto done;
@@ -203,7 +200,7 @@ cmd_modify_vars (struct lexer *lexer, struct dataset *ds)
 		       "names on RENAME subcommand."));
 		  goto done;
 		}
-	      if (!parse_DATA_LIST_vars (lexer, &vm.new_names, 
+	      if (!parse_DATA_LIST_vars (lexer, &vm.new_names,
 					 &prev_nv_1, PV_APPEND))
 		goto done;
 	      if (prev_nv_1 != vm.rename_cnt)
@@ -234,8 +231,8 @@ cmd_modify_vars (struct lexer *lexer, struct dataset *ds)
 
 	  if (already_encountered & 4)
 	    {
-	      msg (SE, _("KEEP subcommand may be given at most once.  It may not"
-		   "be given in conjunction with the DROP subcommand."));
+	      msg (SE, _("KEEP subcommand may be given at most once.  It may "
+                   "not be given in conjunction with the DROP subcommand."));
 	      goto done;
 	    }
 	  already_encountered |= 4;
@@ -294,9 +291,9 @@ cmd_modify_vars (struct lexer *lexer, struct dataset *ds)
 	{
           struct dictionary *temp = dict_clone (dataset_dict (ds));
           int success = rearrange_dict (temp, &vm);
-          if (success) 
+          if (success)
             {
-              /* FIXME: display new dictionary. */ 
+              /* FIXME: display new dictionary. */
             }
           dict_destroy (temp);
 	}
@@ -322,12 +319,12 @@ cmd_modify_vars (struct lexer *lexer, struct dataset *ds)
   if (already_encountered & (1 | 4))
     {
       /* Read the data. */
-      if (!procedure (ds,NULL, NULL)) 
-        goto done; 
+      if (!proc_execute (ds))
+        goto done;
     }
 
   if (!rearrange_dict (dataset_dict (ds), &vm))
-    goto done; 
+    goto done;
 
   ret_code = CMD_SUCCESS;
 
@@ -354,11 +351,11 @@ compare_variables_given_ordering (const void *a_, const void *b_,
   const struct ordering *ordering = ordering_;
 
   int result;
-  if (ordering->positional) 
+  if (ordering->positional)
     {
       size_t a_index = var_get_dict_index (a);
       size_t b_index = var_get_dict_index (b);
-      result = a_index < b_index ? -1 : a_index > b_index; 
+      result = a_index < b_index ? -1 : a_index > b_index;
     }
   else
     result = strcasecmp (var_get_name (a), var_get_name (b));
@@ -378,7 +375,7 @@ struct var_renaming
    var_renaming structures A and B. */
 static int
 compare_var_renaming_by_new_name (const void *a_, const void *b_,
-                                  const void *aux UNUSED) 
+                                  const void *aux UNUSED)
 {
   const struct var_renaming *a = a_;
   const struct var_renaming *b = b_;
@@ -391,7 +388,7 @@ compare_var_renaming_by_new_name (const void *a_, const void *b_,
    otherwise, and issues an error message. */
 static int
 validate_var_modification (const struct dictionary *d,
-                           const struct var_modification *vm) 
+                           const struct var_modification *vm)
 {
   /* Variable reordering can't be a problem, so we don't simulate
      it.  Variable renaming can cause duplicate names, but
@@ -431,14 +428,14 @@ validate_var_modification (const struct dictionary *d,
 
   /* Copy variables into var_renaming array. */
   var_renaming = xnmalloc (keep_cnt, sizeof *var_renaming);
-  for (i = 0; i < keep_cnt; i++) 
+  for (i = 0; i < keep_cnt; i++)
     {
       var_renaming[i].var = keep_vars[i];
       strcpy (var_renaming[i].new_name, var_get_name (keep_vars[i]));
     }
-  
+
   /* Rename variables in var_renaming array. */
-  for (i = 0; i < vm->rename_cnt; i++) 
+  for (i = 0; i < vm->rename_cnt; i++)
     {
       struct variable *const *kv;
       struct var_renaming *vr;
@@ -512,7 +509,7 @@ rearrange_dict (struct dictionary *d, const struct var_modification *vm)
       struct variable *var = dict_lookup_var (d, rename_old_names[i]);
       if (var == NULL)
         continue;
-      
+
       rename_vars[rename_cnt] = var;
       rename_new_names[rename_cnt] = vm->new_names[i];
       rename_cnt++;
