@@ -1,20 +1,18 @@
-/* q2c - parser generator for PSPP procedures.
+/* PSPP - a program for statistical analysis.
    Copyright (C) 1997-9, 2000 Free Software Foundation, Inc.
 
-   This program is free software; you can redistribute it and/or
-   modify it under the terms of the GNU General Public License as
-   published by the Free Software Foundation; either version 2 of the
-   License, or (at your option) any later version.
+   This program is free software: you can redistribute it and/or modify
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation, either version 3 of the License, or
+   (at your option) any later version.
 
-   This program is distributed in the hope that it will be useful, but
-   WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   General Public License for more details.
+   This program is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+   GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
-   02110-1301, USA. */
+   along with this program.  If not, see <http://www.gnu.org/licenses/>. */
 
 #include <assert.h>
 #include <ctype.h>
@@ -152,14 +150,14 @@ static void *
 xmalloc (size_t size)
 {
   void *vp;
-  
+
   if (size == 0)
     return NULL;
-  
+
   vp = malloc (size);
   if (!vp)
     fail ("xmalloc(%lu): %s", (unsigned long) size, VME);
-  
+
   return vp;
 }
 
@@ -173,11 +171,11 @@ xstrdup (const char *s)
 
   assert (s != NULL);
   size = strlen (s) + 1;
-  
+
   t = malloc (size);
   if (!t)
     fail ("xstrdup(%lu): %s", (unsigned long) strlen (s), VME);
-    
+
   memcpy (t, s, size);
   return t;
 }
@@ -192,22 +190,22 @@ get_buffer (void)
 
   if (++cb >= 8)
     cb = 0;
-  
+
   return b[cb];
 }
-   
+
 /* Copies a string to a static buffer, converting it to lowercase in
    the process, and returns a pointer to the static buffer. */
 static char *
 st_lower (const char *s)
 {
   char *p, *cp;
-  
+
   p = cp = get_buffer ();
   while (*s)
     *cp++ = tolower ((unsigned char) (*s++));
   *cp++ = '\0';
-  
+
   return p;
 }
 
@@ -222,7 +220,7 @@ st_upper (const char *s)
   while (*s)
     *cp++ = toupper ((unsigned char) (*s++));
   *cp++ = '\0';
-  
+
   return p;
 }
 
@@ -339,7 +337,7 @@ find_symbol (int x)
   return iter;
 }
 
-#if DUMP_TOKENS 
+#if DUMP_TOKENS
 /* Writes a printable representation of the current token to
    stdout. */
 static void
@@ -369,11 +367,11 @@ lex_get (void)
       cp = skip_ws (cp);
       if (*cp != '\0')
 	break;
-      
+
       if (!get_line ())
 	fail ("%s: Unexpected end of file.", ifn);
     }
-  
+
   if (*cp == '"')
     {
       char *dest = tokstr;
@@ -406,11 +404,11 @@ lex_get (void)
     }
   else
     token = *cp++;
-  
+
 #if DUMP_TOKENS
   dump_token ();
 #endif
-  
+
   return token;
 }
 
@@ -472,7 +470,8 @@ enum
   {
     VAL_NONE,	/* No value. */
     VAL_INT,	/* Integer value. */
-    VAL_DBL	/* Floating point value. */
+    VAL_DBL,	/* Floating point value. */
+    VAL_STRING  /* String value. */
   };
 
 /* For those specifiers with values, the syntax of those values. */
@@ -511,7 +510,7 @@ struct specifier
 
     setting *def;	/* Default setting. */
     setting *omit_kw;	/* Setting for which the keyword can be omitted. */
-    
+
     int index;		/* Next array index. */
   };
 
@@ -550,7 +549,7 @@ struct subcommand
     int narray;			/* Index of next array element. */
     const char *prefix;		/* Prefix for variable and constant names. */
     specifier *spec;		/* Array of specifiers. */
-    
+
     /* SBC_STRING and SBC_INT only. */
     char *restriction;		/* Expression restricting string length. */
     char *message;		/* Error message. */
@@ -601,7 +600,7 @@ static void
 parse_setting (setting *s, specifier *spec)
 {
   s->parent = spec;
-  
+
   if (match_token ('*'))
     {
       if (spec->omit_kw)
@@ -609,7 +608,7 @@ parse_setting (setting *s, specifier *spec)
       else
 	spec->omit_kw = s;
     }
-  
+
   if (match_token ('!'))
     {
       if (spec->def)
@@ -617,7 +616,7 @@ parse_setting (setting *s, specifier *spec)
       else
 	spec->def = s;
     }
-  
+
   force_id ();
   s->specname = xstrdup (tokstr);
   s->con = add_symbol (s->specname, 0, 0);
@@ -637,20 +636,22 @@ parse_setting (setting *s, specifier *spec)
 	s->valtype = VT_PLAIN;
 
       s->optvalue = match_token ('*');
-      
+
       if (match_id ("N"))
 	s->value = VAL_INT;
       else if (match_id ("D"))
 	s->value = VAL_DBL;
+      else if (match_id ("S"))
+        s->value = VAL_STRING;
       else
-	error ("`n' or `d' expected.");
-      
+	error ("`n', `d', or `s' expected.");
+
       skip_token (':');
-      
+
       force_id ();
       s->valname = xstrdup (tokstr);
       lex_get ();
-      
+
       if (token == ',')
 	{
 	  lex_get ();
@@ -660,7 +661,7 @@ parse_setting (setting *s, specifier *spec)
 	}
       else
 	s->restriction = NULL;
-      
+
       if (s->valtype == VT_PAREN)
 	skip_token (')');
     }
@@ -682,7 +683,7 @@ parse_specifier (specifier *spec, subcommand *sbc)
       spec->varname = xstrdup (st_lower (tokstr));
       lex_get ();
     }
-  
+
   /* Handle array elements. */
   if (token != ':')
     {
@@ -698,20 +699,20 @@ parse_specifier (specifier *spec, subcommand *sbc)
       return;
     }
   skip_token (':');
-  
-  if ( sbc->type == SBC_ARRAY && token == T_ID ) 
+
+  if ( sbc->type == SBC_ARRAY && token == T_ID )
     {
 	spec->varname = xstrdup (st_lower (tokstr));
 	spec->index = sbc->narray;
 	sbc->narray++;
     }
-    
-  
-  
+
+
+
   /* Parse all the settings. */
   {
     setting **s = &spec->s;
-    
+
     for (;;)
       {
 	*s = xmalloc (sizeof **s);
@@ -736,7 +737,7 @@ parse_specifiers (subcommand *sbc)
       *spec = NULL;
       return;
     }
-  
+
   for (;;)
     {
       *spec = xmalloc (sizeof **spec);
@@ -770,7 +771,7 @@ parse_subcommand (subcommand *sbc)
   force_id ();
   sbc->name = xstrdup (tokstr);
   lex_get ();
-  
+
   sbc->narray = 0;
   sbc->type = SBC_PLAIN;
   sbc->spec = NULL;
@@ -781,10 +782,10 @@ parse_subcommand (subcommand *sbc)
       force_id ();
       sbc->prefix = xstrdup (st_lower (tokstr));
       lex_get ();
-      
+
       skip_token (']');
       skip_token ('=');
-      
+
       sbc->type = SBC_ARRAY;
       parse_specifiers (sbc);
 
@@ -796,12 +797,12 @@ parse_subcommand (subcommand *sbc)
 	  force_id ();
 	  sbc->prefix = xstrdup (st_lower (tokstr));
 	  lex_get ();
-	  
+
 	  skip_token (')');
 	}
       else
 	sbc->prefix = "";
-      
+
       skip_token ('=');
 
       if (match_id ("VAR"))
@@ -813,7 +814,7 @@ parse_subcommand (subcommand *sbc)
 	      force_string ();
 	      sbc->message = xstrdup (tokstr);
 	      lex_get();
-	      
+
 	      skip_token (')');
 	    }
 	  else sbc->message = NULL;
@@ -823,7 +824,7 @@ parse_subcommand (subcommand *sbc)
       else if (match_id ("INTEGER"))
        {
 	sbc->type = match_id ("LIST") ? SBC_INT_LIST : SBC_INT;
-        if ( token == T_STRING) 
+        if ( token == T_STRING)
          {
 	      sbc->restriction = xstrdup (tokstr);
 	      lex_get ();
@@ -879,7 +880,7 @@ void
 parse_subcommands (void)
 {
   subcommand **sbc = &subcommands;
-  
+
   for (;;)
     {
       *sbc = xmalloc (sizeof **sbc);
@@ -922,7 +923,7 @@ dump (int indention, const char *format, ...)
 
   if (indention < 0)
     indent += BASE_INDENT * indention;
-  
+
   oln++;
   va_start (args, format);
   for (i = 0; i < indent; i++)
@@ -942,7 +943,7 @@ dump_specifier_vars (const specifier *spec, const subcommand *sbc)
 {
   if (spec->varname)
     dump (0, "long %s%s;", sbc->prefix, spec->varname);
-  
+
   {
     setting *s;
 
@@ -952,8 +953,11 @@ dump_specifier_vars (const specifier *spec, const subcommand *sbc)
 	  {
 	    const char *typename;
 
-	    assert (s->value == VAL_INT || s->value == VAL_DBL);
-	    typename = s->value == VAL_INT ? "long" : "double";
+	    assert (s->value == VAL_INT || s->value == VAL_DBL
+                    || s->value == VAL_STRING);
+	    typename = (s->value == VAL_INT ? "long"
+                        : s->value == VAL_DBL ? "double"
+                        : "char *");
 
 	    dump (0, "%s %s%s;", typename, sbc->prefix, st_lower (s->valname));
 	  }
@@ -993,7 +997,7 @@ make_identifier (const char *name)
     else
       *cp++ = '_';
   *cp = '\0';
-  
+
   return p;
 }
 
@@ -1003,6 +1007,8 @@ dump_declarations (void)
 {
   indent = 0;
 
+  dump (0, "struct dataset;");
+
   /* Write out enums for all the identifiers in the symbol table. */
   {
     int f, k;
@@ -1026,7 +1032,7 @@ dump_declarations (void)
 	    buf = xmalloc (1024);
 	  else
 	    dump (0, buf);
-	  
+
 	  if (k)
 	    sprintf (buf, "%s%s,", st_upper (prefix), sym->name);
 	  else
@@ -1063,7 +1069,7 @@ dump_declarations (void)
       if (sbc->type == SBC_ARRAY && sbc->narray)
 	{
 	  dump (0, "/* Array indices for %s subcommand. */", sbc->name);
-	  
+
 	  dump (1, "enum");
 	  dump (1, "{");
 
@@ -1096,7 +1102,7 @@ dump_declarations (void)
 
 	if (sbc != subcommands)
 	  dump (0, nullstr);
-	
+
 	dump (0, "/* %s subcommand. */", sbc->name);
 	dump (0, "int sbc_%s;", st_lower (sbc->name));
 
@@ -1106,7 +1112,7 @@ dump_declarations (void)
 	  case SBC_PLAIN:
 	    {
 	      specifier *spec;
-	    
+
 	      for (spec = sbc->spec; spec; spec = spec->next)
 		{
 		  if (spec->s == 0)
@@ -1116,8 +1122,8 @@ dump_declarations (void)
 			      spec->varname);
 		      else if (f == 0)
 			{
-			  dump (0, "int a_%s[%s%scount];", 
-				st_lower (sbc->name), 
+			  dump (0, "int a_%s[%s%scount];",
+				st_lower (sbc->name),
 				st_upper (prefix),
 				st_upper (sbc->prefix)
 				);
@@ -1226,7 +1232,7 @@ dump_specifier_init (const specifier *spec, const subcommand *sbc)
 	strcpy (s, "-1");
       dump (0, "p->%s%s = %s;", sbc->prefix, spec->varname, s);
     }
-  
+
   {
     setting *s;
 
@@ -1236,8 +1242,11 @@ dump_specifier_init (const specifier *spec, const subcommand *sbc)
 	  {
 	    const char *init;
 
-	    assert (s->value == VAL_INT || s->value == VAL_DBL);
-	    init = s->value == VAL_INT ? "NOT_LONG" : "SYSMIS";
+	    assert (s->value == VAL_INT || s->value == VAL_DBL
+                    || s->value == VAL_STRING);
+	    init = (s->value == VAL_INT ? "NOT_LONG"
+                    : s->value == VAL_DBL ? "SYSMIS"
+                    : "NULL");
 
 	    dump (0, "p->%s%s = %s;", sbc->prefix, st_lower (s->valname), init);
 	  }
@@ -1252,24 +1261,24 @@ dump_vars_init (int persistent)
   /* Loop through all the subcommands. */
   {
     subcommand *sbc;
-    
+
     for (sbc = subcommands; sbc; sbc = sbc->next)
       {
 	int f = 0;
-	
+
 	dump (0, "p->sbc_%s = 0;", st_lower (sbc->name));
-	if ( ! persistent ) 
+	if ( ! persistent )
 	  {
 	    switch (sbc->type)
 	      {
 	      case SBC_INT_LIST:
-		break;
-
 	      case SBC_DBL_LIST:
 		dump (1, "{");
 		dump (0, "int i;");
 		dump (1, "for (i = 0; i < MAXLISTS; ++i)");
-		dump (0, "subc_list_double_create(&p->dl_%s[i]) ;",
+		dump (0, "subc_list_%s_create(&p->%cl_%s[i]) ;",
+                      sbc->type == SBC_INT_LIST ? "int" : "double",
+                      sbc->type == SBC_INT_LIST ? 'i' : 'd',
 		      st_lower (sbc->name)
 		      );
 		dump (-2, "}");
@@ -1286,12 +1295,12 @@ dump_vars_init (int persistent)
 	      case SBC_CUSTOM:
 		/* nothing */
 		break;
-	    
+
 	      case SBC_PLAIN:
 	      case SBC_ARRAY:
 		{
 		  specifier *spec;
-	    
+
 		  for (spec = sbc->spec; spec; spec = spec->next)
 		    if (spec->s == NULL)
 		      {
@@ -1315,7 +1324,7 @@ dump_vars_init (int persistent)
 		dump (0, "p->%sv_%s = NULL;",
 		      st_lower (sbc->prefix), st_lower (sbc->name));
 		break;
-	    
+
 	      case SBC_VAR:
 		dump (0, "p->%sv_%s = NULL;",
 		      st_lower (sbc->prefix), st_lower (sbc->name));
@@ -1353,7 +1362,7 @@ make_match (const char *t)
 
   while (*t == '_')
     t++;
-      
+
   if (is_keyword (t))
     sprintf (s, "lex_match (lexer, T_%s)", t);
   else if (!strcmp (t, "ON") || !strcmp (t, "YES"))
@@ -1366,7 +1375,7 @@ make_match (const char *t)
     sprintf (s, "lex_match_int (lexer, %s)", t);
   else
     sprintf (s, "lex_match_id (lexer, \"%s\")", t);
-  
+
   return s;
 }
 
@@ -1383,7 +1392,7 @@ dump_specifier_parse (const specifier *spec, const subcommand *sbc)
   if (spec->omit_kw && spec->omit_kw->parent->next)
     error ("Default specifier is not in last specifier in `%s' "
 	   "subcommand.", sbc->name);
-  
+
   for (s = spec->s; s; s = s->next)
     {
       int first = spec == sbc->spec && s == spec->s;
@@ -1411,13 +1420,13 @@ dump_specifier_parse (const specifier *spec, const subcommand *sbc)
 	{
 	  if (spec->omit_kw != s)
 	    dump (1, "{");
-	  
+
 	  if (spec->varname)
 	    {
 	      dump (0, "p->%s%s = %s%s;", sbc->prefix, spec->varname,
 		    st_upper (prefix), find_symbol (s->con)->name);
 
-	      if ( sbc->type == SBC_ARRAY ) 
+	      if ( sbc->type == SBC_ARRAY )
 		dump (0, "p->a_%s[%s%s%s] = 1;",
 		      st_lower (sbc->name),
 		      st_upper (prefix), st_upper (sbc->prefix),
@@ -1457,7 +1466,7 @@ dump_specifier_parse (const specifier *spec, const subcommand *sbc)
 	      dump (-1, "p->%s%s = lex_integer (lexer);",
 		    sbc->prefix, st_lower (s->valname));
 	    }
-	  else
+	  else if (s->value == VAL_DBL)
 	    {
 	      dump (1, "if (!lex_is_number (lexer))");
 	      dump (1, "{");
@@ -1469,7 +1478,23 @@ dump_specifier_parse (const specifier *spec, const subcommand *sbc)
 	      dump (-1, "p->%s%s = lex_tokval (lexer);", sbc->prefix,
 		    st_lower (s->valname));
 	    }
-	  
+          else if (s->value == VAL_STRING)
+            {
+              dump (1, "if (lex_token (lexer) != T_ID "
+                    "&& lex_token (lexer) != T_STRING)");
+              dump (1, "{");
+              dump (0, "msg (SE, _(\"%s specifier of %s subcommand "
+                    "requires a string argument.\"));",
+		    s->specname, sbc->name);
+	      dump (0, "goto lossage;");
+	      dump (-1, "}");
+              dump (-1, "free (p->%s%s);", sbc->prefix, st_lower (s->valname));
+              dump (0, "p->%s%s = xstrdup (ds_cstr (lex_tokstr (lexer)));",
+                    sbc->prefix, st_lower (s->valname));
+            }
+          else
+            abort ();
+
 	  if (s->restriction)
 	    {
 	      {
@@ -1483,7 +1508,7 @@ dump_specifier_parse (const specifier *spec, const subcommand *sbc)
 		free (str);
 		free (str2);
 	      }
-	      
+
 	      dump (1, "{");
 	      dump (0, "msg (SE, _(\"Bad argument for %s "
 		    "specifier of %s subcommand.\"));",
@@ -1492,9 +1517,9 @@ dump_specifier_parse (const specifier *spec, const subcommand *sbc)
 	      dump (-1, "}");
 	      outdent ();
 	    }
-	  
+
 	  dump (0, "lex_get (lexer);");
-	  
+
 	  if (s->valtype == VT_PAREN)
 	    {
 	      dump (1, "if (!lex_match (lexer, ')'))");
@@ -1511,11 +1536,11 @@ dump_specifier_parse (const specifier *spec, const subcommand *sbc)
 		  outdent ();
 		}
 	    }
-	  
+
 	  if (s != spec->omit_kw)
 	    dump (-1, "}");
 	}
-      
+
       if (s == spec->omit_kw)
 	{
 	  dump (-1, "}");
@@ -1535,7 +1560,7 @@ dump_subcommand (const subcommand *sbc)
 
       dump (1, "while (lex_token (lexer) != '/' && lex_token (lexer) != '.')");
       dump (1, "{");
-      
+
       {
 	specifier *spec;
 
@@ -1560,7 +1585,7 @@ dump_subcommand (const subcommand *sbc)
 	      }
 	  }
       }
-      
+
       {
 	specifier *spec;
 	setting *s;
@@ -1647,7 +1672,7 @@ dump_subcommand (const subcommand *sbc)
     {
       dump (1, "if (!lex_force_num (lexer))");
       dump (0, "goto lossage;");
-      dump (-1, "p->n_%s[p->sbc_%s - 1] = lex_number (lexer);", 
+      dump (-1, "p->n_%s[p->sbc_%s - 1] = lex_number (lexer);",
 	    st_lower (sbc->name), st_lower (sbc->name) );
       dump (0, "lex_get(lexer);");
     }
@@ -1663,9 +1688,9 @@ dump_subcommand (const subcommand *sbc)
        {
 	  char buf[1024];
 	  dump (1, "if (!(%s))", sbc->restriction);
-	  dump (1, "{"); 
+	  dump (1, "{");
           sprintf(buf,sbc->message,sbc->name);
-	  if ( sbc->translatable ) 
+	  if ( sbc->translatable )
 		  dump (0, "msg (SE, gettext(\"%s\"));",buf);
 	  else
 		  dump (0, "msg (SE, \"%s\");",buf);
@@ -1683,7 +1708,7 @@ dump_subcommand (const subcommand *sbc)
       dump (-1, "p->n_%s = lex_integer (lexer);", st_lower (sbc->name));
       dump (0, "lex_match (lexer, ')');");
     }
-  else if (sbc->type == SBC_DBL_LIST)
+  else if (sbc->type == SBC_DBL_LIST || sbc->type == SBC_INT_LIST)
     {
       dump (0, "if ( p->sbc_%s > MAXLISTS)",st_lower(sbc->name));
       dump (1, "{");
@@ -1699,9 +1724,10 @@ dump_subcommand (const subcommand *sbc)
       dump (0, "goto lossage;");
       dump (-1,"}");
 
-      dump (0, "subc_list_double_push (&p->dl_%s[p->sbc_%s-1], lex_number (lexer));", 
-	    st_lower (sbc->name), st_lower (sbc->name)
-	    );
+      dump (0, "subc_list_%s_push (&p->%cl_%s[p->sbc_%s-1], lex_number (lexer));",
+            sbc->type == SBC_INT_LIST ? "int" : "double",
+            sbc->type == SBC_INT_LIST ? 'i' : 'd',
+            st_lower (sbc->name), st_lower (sbc->name));
 
       dump (0, "lex_get (lexer);");
       dump (-1,"}");
@@ -1794,7 +1820,7 @@ dump_parser (int persistent)
       dump (-1, "}");
       outdent ();
     }
-  
+
   {
     subcommand *sbc;
 
@@ -1839,7 +1865,7 @@ dump_parser (int persistent)
   outdent ();
 
 
-  
+
   dump (1, "if (!lex_match (lexer, '/'))");
   dump (0, "break;");
   dump (-2, "}");
@@ -1861,7 +1887,7 @@ dump_parser (int persistent)
     for (sbc = subcommands; sbc; sbc = sbc->next)
       {
 
-	if ( sbc->arity == ARITY_ONCE_EXACTLY ) 
+	if ( sbc->arity == ARITY_ONCE_EXACTLY )
 	  {
 	    dump (0, "if ( 0 == p->sbc_%s)", st_lower (sbc->name));
 	    dump (1, "{");
@@ -1907,16 +1933,12 @@ dump_free (int persistent)
   indent = 0;
 
   used = 0;
-  if ( ! persistent ) 
+  if ( ! persistent )
     {
       for (sbc = subcommands; sbc; sbc = sbc->next)
-	{
-	if (sbc->type == SBC_STRING)
-	  used = 1;
-	if (sbc->type == SBC_DBL_LIST)
-	  used = 1;
-	}
-
+        used = (sbc->type == SBC_STRING
+                || sbc->type == SBC_DBL_LIST
+                || sbc->type == SBC_INT_LIST);
     }
 
   dump (0, "static void");
@@ -1924,12 +1946,12 @@ dump_free (int persistent)
 	make_identifier (cmdname), used ? "" : " UNUSED");
   dump (1, "{");
 
-  if ( ! persistent ) 
+  if ( ! persistent )
     {
 
       for (sbc = subcommands; sbc; sbc = sbc->next)
 	{
-	  switch (sbc->type) 
+	  switch (sbc->type)
 	    {
             case SBC_VARLIST:
 	      dump (0, "free (p->v_%s);", st_lower (sbc->name));
@@ -1938,13 +1960,28 @@ dump_free (int persistent)
 	      dump (0, "free (p->s_%s);", st_lower (sbc->name));
 	      break;
 	    case SBC_DBL_LIST:
+	    case SBC_INT_LIST:
               dump (0, "{");
 	      dump (1, "int i;");
 	      dump (2, "for(i = 0; i < MAXLISTS ; ++i)");
-	      dump (1, "subc_list_double_destroy(&p->dl_%s[i]);", st_lower (sbc->name));
+	      dump (1, "subc_list_%s_destroy(&p->%cl_%s[i]);",
+                    sbc->type == SBC_INT_LIST ? "int" : "double",
+                    sbc->type == SBC_INT_LIST ? 'i' : 'd',
+                    st_lower (sbc->name));
               dump (0, "}");
 	      outdent();
 	      break;
+            case SBC_PLAIN:
+              {
+                specifier *spec;
+                setting *s;
+
+                for (spec = sbc->spec; spec; spec = spec->next)
+                  for (s = spec->s; s; s = s->next)
+                    if (s->value == VAL_STRING)
+                      dump (0, "free (p->%s%s);",
+                            sbc->prefix, st_lower (s->valname));
+              }
 	    default:
 	      break;
 	    }
@@ -1964,7 +2001,7 @@ recognize_directive (void)
 {
   static char directive[16];
   char *sp, *ep;
-  
+
   sp = skip_ws (buf);
   if (strncmp (sp, "/*", 2))
     return NULL;
@@ -1983,7 +2020,7 @@ recognize_directive (void)
   directive[ep - sp] = '\0';
   return directive;
 }
-  
+
 int
 main (int argc, char *argv[])
 {
@@ -2018,7 +2055,7 @@ main (int argc, char *argv[])
 	  dump (0, "%s", buf);
 	  continue;
 	}
-      
+
       dump (0, "#line %d \"%s\"", oln + 1, ofn);
       if (!strcmp (directive, "specification"))
 	{
@@ -2042,6 +2079,7 @@ main (int argc, char *argv[])
 	  dump (0, "#include <language/lexer/lexer.h>");
 	  dump (0, "#include <language/lexer/variable-parser.h>");
           dump (0, "#include <data/settings.h>");
+	  dump (0, "#include <libpspp/magic.h>");
 	  dump (0, "#include <libpspp/str.h>");
           dump (0, "#include <language/lexer/subcommand-list.h>");
 	  dump (0, "#include <data/variable.h>");
@@ -2056,12 +2094,12 @@ main (int argc, char *argv[])
       else if (!strcmp (directive, "functions"))
 	{
 	  dump_parser (0);
-	  dump_free (0); 
+	  dump_free (0);
 	}
       else if (!strcmp (directive, "_functions"))
 	{
 	  dump_parser (1);
-	  dump_free (1); 
+	  dump_free (1);
 	}
       else
 	error ("unknown directive `%s'", directive);
