@@ -1,25 +1,24 @@
-/* PSPP - computes sample statistics.
+/* PSPP - a program for statistical analysis.
    Copyright (C) 1997-9, 2000 Free Software Foundation, Inc.
 
-   This program is free software; you can redistribute it and/or
-   modify it under the terms of the GNU General Public License as
-   published by the Free Software Foundation; either version 2 of the
-   License, or (at your option) any later version.
+   This program is free software: you can redistribute it and/or modify
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation, either version 3 of the License, or
+   (at your option) any later version.
 
-   This program is distributed in the hope that it will be useful, but
-   WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   General Public License for more details.
+   This program is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+   GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
-   02110-1301, USA. */
+   along with this program.  If not, see <http://www.gnu.org/licenses/>. */
 
 #include <config.h>
 #include <stdlib.h>
 
 #include <data/case.h>
+#include <data/casereader.h>
 #include <data/dictionary.h>
 #include <data/procedure.h>
 #include <data/transformations.h>
@@ -41,7 +40,7 @@
 /* FIXME: Implement PRINT subcommand. */
 
 /* An AUTORECODE variable's original value. */
-union arc_value 
+union arc_value
   {
     double f;                   /* Numeric. */
     char *c;                    /* Short or long string. */
@@ -71,14 +70,14 @@ struct autorecode_trns
   };
 
 /* Descending or ascending sort order. */
-enum direction 
+enum direction
   {
     ASCENDING,
     DESCENDING
   };
 
 /* AUTORECODE data. */
-struct autorecode_pgm 
+struct autorecode_pgm
   {
     const struct variable **src_vars;    /* Source variables. */
     char **dst_names;              /* Target variable names. */
@@ -103,7 +102,8 @@ int
 cmd_autorecode (struct lexer *lexer, struct dataset *ds)
 {
   struct autorecode_pgm arc;
-  struct ccase *c;
+  struct casereader *input;
+  struct ccase c;
   size_t dst_cnt;
   size_t i;
   bool ok;
@@ -120,7 +120,7 @@ cmd_autorecode (struct lexer *lexer, struct dataset *ds)
 
   lex_match_id (lexer, "VARIABLES");
   lex_match (lexer, '=');
-  if (!parse_variables_const (lexer, dataset_dict (ds), &arc.src_vars, 
+  if (!parse_variables_const (lexer, dataset_dict (ds), &arc.src_vars,
 			      &arc.var_cnt,
                         PV_NO_DUPLICATE))
     goto lossage;
@@ -188,16 +188,16 @@ cmd_autorecode (struct lexer *lexer, struct dataset *ds)
                                       hash_numeric_value, NULL, NULL);
    }
 
-  proc_open (ds);
-  while (proc_read (ds, &c))
+  input = proc_open (ds);
+  for (; casereader_read (input, &c); case_destroy (&c))
     for (i = 0; i < arc.var_cnt; i++)
       {
         union arc_value v, *vp, **vpp;
 
         if (var_is_numeric (arc.src_vars[i]))
-          v.f = case_num (c, arc.src_vars[i]);
+          v.f = case_num (&c, arc.src_vars[i]);
         else
-          v.c = (char *) case_str (c, arc.src_vars[i]);
+          v.c = (char *) case_str (&c, arc.src_vars[i]);
 
         vpp = (union arc_value **) hsh_probe (arc.src_values[i], &v);
         if (*vpp == NULL)
@@ -211,7 +211,8 @@ cmd_autorecode (struct lexer *lexer, struct dataset *ds)
             *vpp = vp;
           }
       }
-  ok = proc_close (ds);
+  ok = casereader_destroy (input);
+  ok = proc_commit (ds) && ok;
 
   for (i = 0; i < arc.var_cnt; i++)
     arc.dst_vars[i] = dict_create_var_assert (dataset_dict (ds),
@@ -227,19 +228,19 @@ lossage:
 }
 
 static void
-arc_free (struct autorecode_pgm *arc) 
+arc_free (struct autorecode_pgm *arc)
 {
   free (arc->src_vars);
-  if (arc->dst_names != NULL) 
+  if (arc->dst_names != NULL)
     {
       size_t i;
-      
+
       for (i = 0; i < arc->var_cnt; i++)
         free (arc->dst_names[i]);
       free (arc->dst_names);
     }
   free (arc->dst_vars);
-  if (arc->src_values != NULL) 
+  if (arc->src_values != NULL)
     {
       size_t i;
 
@@ -283,7 +284,7 @@ recode (struct dataset *ds, const struct autorecode_pgm *arc)
 	{
 	  struct arc_item *item = pool_alloc (trns->pool, sizeof *item);
           union arc_value *vp = *p;
-          
+
 	  if (var_is_numeric (arc->src_vars[i]))
             item->from.f = vp->f;
           else
@@ -293,7 +294,7 @@ recode (struct dataset *ds, const struct autorecode_pgm *arc)
 	  hsh_force_insert (spec->items, item);
 	}
     }
-  add_transformation (ds, 
+  add_transformation (ds,
 		      autorecode_trns_proc, autorecode_trns_free, trns);
 }
 
@@ -351,7 +352,7 @@ hash_alpha_value (const void *a_, const void *v_)
 {
   const union arc_value *a = a_;
   const struct variable *v = v_;
-  
+
   return hsh_hash_bytes (a->c, var_get_width (v));
 }
 
