@@ -1,20 +1,18 @@
-/* PSPP - computes sample statistics.
+/* PSPP - a program for statistical analysis.
    Copyright (C) 1997-9, 2000 Free Software Foundation, Inc.
 
-   This program is free software; you can redistribute it and/or
-   modify it under the terms of the GNU General Public License as
-   published by the Free Software Foundation; either version 2 of the
-   License, or (at your option) any later version.
+   This program is free software: you can redistribute it and/or modify
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation, either version 3 of the License, or
+   (at your option) any later version.
 
-   This program is distributed in the hope that it will be useful, but
-   WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   General Public License for more details.
+   This program is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+   GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
-   02110-1301, USA. */
+   along with this program.  If not, see <http://www.gnu.org/licenses/>. */
 
 #include "config.h"
 
@@ -27,9 +25,9 @@
 #include <sys/types.h>
 #endif
 
-#include <data/case-sink.h>
-#include <data/case-source.h>
 #include <data/case.h>
+#include <data/casereader.h>
+#include <data/casereader-provider.h>
 #include <data/dictionary.h>
 #include <data/procedure.h>
 #include <data/settings.h>
@@ -42,7 +40,6 @@
 #include <libpspp/array.h>
 #include <libpspp/assertion.h>
 #include <libpspp/message.h>
-#include <libpspp/message.h>
 #include <libpspp/misc.h>
 #include <libpspp/pool.h>
 #include <libpspp/str.h>
@@ -61,16 +58,12 @@ struct varname
   };
 
 /* Represents a FLIP input program. */
-struct flip_pgm 
+struct flip_pgm
   {
     struct pool *pool;          /* Pool containing FLIP data. */
     const struct variable **var;      /* Variables to transpose. */
-    int *idx_to_fv;             /* var[]->index to compacted sink case fv. */
     size_t var_cnt;             /* Number of elements in `var'. */
     int case_cnt;               /* Pre-flip case count. */
-    size_t case_size;           /* Post-flip bytes per case. */
-
-    union value *output_buf;            /* Case output buffer. */
 
     struct variable *new_names; /* Variable containing new variable names. */
     struct varname *new_names_head; /* First new variable. */
@@ -82,22 +75,23 @@ struct flip_pgm
     bool error;                 /* Error reading temporary file? */
   };
 
-static void destroy_flip_pgm (struct flip_pgm *);
-static struct case_sink *flip_sink_create (struct dataset *ds, struct flip_pgm *);
-static struct case_source *flip_source_create (struct flip_pgm *);
-static bool flip_file (struct flip_pgm *);
-static int build_dictionary (struct dictionary *, struct flip_pgm *);
+static const struct casereader_class flip_casereader_class;
 
-static const struct case_source_class flip_source_class;
-static const struct case_sink_class flip_sink_class;
+static void destroy_flip_pgm (struct flip_pgm *);
+static bool flip_file (struct flip_pgm *);
+static bool build_dictionary (struct dictionary *, struct flip_pgm *);
+static bool write_flip_case (struct flip_pgm *, const struct ccase *);
 
 /* Parses and executes FLIP. */
 int
 cmd_flip (struct lexer *lexer, struct dataset *ds)
 {
-  struct flip_pgm *flip;
-  struct case_sink *sink;
   struct dictionary *dict = dataset_dict (ds);
+  struct flip_pgm *flip;
+  struct casereader *input, *reader;
+  union value *output_buf;
+  struct ccase c;
+  size_t i;
   bool ok;
 
   if (proc_make_temporary_transformations_permanent (ds))
@@ -106,8 +100,6 @@ cmd_flip (struct lexer *lexer, struct dataset *ds)
 
   flip = pool_create_container (struct flip_pgm, pool);
   flip->var = NULL;
-  flip->idx_to_fv = dict_get_compacted_dict_index_to_case_index (dict);
-  pool_register (flip->pool, free, flip->idx_to_fv);
   flip->var_cnt = 0;
   flip->case_cnt = 0;
   flip->new_names = NULL;
@@ -123,7 +115,7 @@ cmd_flip (struct lexer *lexer, struct dataset *ds)
     {
       lex_match (lexer, '=');
       if (!parse_variables_const (lexer, dict, &flip->var, &flip->var_cnt,
-                            PV_NO_DUPLICATE))
+                                  PV_NO_DUPLICATE))
 	goto error;
       lex_match (lexer, '/');
     }
@@ -144,8 +136,6 @@ cmd_flip (struct lexer *lexer, struct dataset *ds)
 
   if (flip->new_names)
     {
-      size_t i;
-      
       for (i = 0; i < flip->var_cnt; i++)
 	if (flip->var[i] == flip->new_names)
 	  {
@@ -155,20 +145,44 @@ cmd_flip (struct lexer *lexer, struct dataset *ds)
 	  }
     }
 
+  output_buf = pool_nalloc (flip->pool, flip->var_cnt, sizeof *output_buf);
+
+  flip->file = pool_tmpfile (flip->pool);
+  if (flip->file == NULL)
+    {
+      msg (SE, _("Could not create temporary file for FLIP."));
+      goto error;
+    }
+
+  /* Write variable names as first case. */
+  for (i = 0; i < flip->var_cnt; i++)
+    buf_copy_str_rpad (output_buf[i].s, MAX_SHORT_STRING,
+                       var_get_name (flip->var[i]));
+  if (fwrite (output_buf, sizeof *output_buf,
+              flip->var_cnt, flip->file) != (size_t) flip->var_cnt)
+    {
+      msg (SE, _("Error writing FLIP file: %s."), strerror (errno));
+      goto error;
+    }
+
+  flip->case_cnt = 1;
+
   /* Read the active file into a flip_sink. */
-  flip->case_cnt = 0;
-  proc_make_temporary_transformations_permanent (ds);
-  sink = flip_sink_create (ds, flip);
-  if (sink == NULL)
-    goto error;
-  proc_set_sink (ds, sink);
-  flip->new_names_tail = NULL;
-  ok = procedure (ds,NULL, NULL);
+  proc_discard_output (ds);
+
+  input = proc_open (ds);
+  while (casereader_read (input, &c))
+    {
+      write_flip_case (flip, &c);
+      case_destroy (&c);
+    }
+  ok = casereader_destroy (input);
+  ok = proc_commit (ds) && ok;
 
   /* Flip the data we read. */
-  if (!flip_file (flip)) 
+  if (!ok || !flip_file (flip))
     {
-      discard_variables (ds);
+      proc_discard_active_file (ds);
       goto error;
     }
 
@@ -176,15 +190,16 @@ cmd_flip (struct lexer *lexer, struct dataset *ds)
   dict_clear (dict);
   if (!build_dictionary (dict, flip))
     {
-      discard_variables (ds);
+      proc_discard_active_file (ds);
       goto error;
     }
-  flip->case_size = dict_get_case_size (dict);
 
   /* Set up flipped data for reading. */
-  proc_set_source (ds, flip_source_create (flip));
-
-  return ok ? lex_end_of_command (lexer) : CMD_CASCADING_FAILURE;
+  reader = casereader_create_sequential (NULL, dict_get_next_value_idx (dict),
+                                         flip->var_cnt,
+                                         &flip_casereader_class, flip);
+  proc_set_active_file_data (ds, reader);
+  return lex_end_of_command (lexer);
 
  error:
   destroy_flip_pgm (flip);
@@ -193,7 +208,7 @@ cmd_flip (struct lexer *lexer, struct dataset *ds)
 
 /* Destroys FLIP. */
 static void
-destroy_flip_pgm (struct flip_pgm *flip) 
+destroy_flip_pgm (struct flip_pgm *flip)
 {
   if (flip != NULL)
     pool_destroy (flip->pool);
@@ -213,7 +228,7 @@ make_new_var (struct dictionary *dict, char name[])
 
   /* Fix invalid characters. */
   for (cp = name; *cp && cp < name + SHORT_NAME_LEN; cp++)
-    if (cp == name) 
+    if (cp == name)
       {
         if (!lex_is_id1 (*cp) || *cp == '$')
           *cp = 'V';
@@ -221,11 +236,11 @@ make_new_var (struct dictionary *dict, char name[])
     else
       {
         if (!lex_is_idn (*cp))
-          *cp = '_'; 
+          *cp = '_';
       }
   *cp = '\0';
   str_uppercase (name);
-  
+
   if (dict_create_var (dict, name, 0))
     return 1;
 
@@ -251,7 +266,7 @@ make_new_var (struct dictionary *dict, char name[])
 }
 
 /* Make a new dictionary for all the new variable names. */
-static int
+static bool
 build_dictionary (struct dictionary *dict, struct flip_pgm *flip)
 {
   dict_create_var_assert (dict, "CASE_LBL", 8);
@@ -259,14 +274,14 @@ build_dictionary (struct dictionary *dict, struct flip_pgm *flip)
   if (flip->new_names_head == NULL)
     {
       int i;
-      
+
       if (flip->case_cnt > 99999)
 	{
 	  msg (SE, _("Cannot create more than 99999 variable names."));
-	  return 0;
+	  return false;
 	}
-      
-      for (i = 0; i < flip->case_cnt; i++)
+
+      for (i = 0; i < flip->case_cnt - 1; i++)
 	{
           struct variable *v;
 	  char s[SHORT_NAME_LEN + 1];
@@ -281,66 +296,28 @@ build_dictionary (struct dictionary *dict, struct flip_pgm *flip)
 
       for (v = flip->new_names_head; v; v = v->next)
         if (!make_new_var (dict, v->name))
-          return 0;
-    }
-  
-  return 1;
-}
-     
-/* Creates a flip sink based on FLIP. */
-static struct case_sink *
-flip_sink_create (struct dataset *ds, struct flip_pgm *flip) 
-{
-  size_t i;
-
-  flip->output_buf = pool_nalloc (flip->pool,
-                                  flip->var_cnt, sizeof *flip->output_buf);
-
-  flip->file = pool_tmpfile (flip->pool);
-  if (flip->file == NULL)
-    {
-      msg (SE, _("Could not create temporary file for FLIP: %s."),
-           strerror (errno));
-      return NULL;
+          return false;
     }
 
-  /* Write variable names as first case. */
-  for (i = 0; i < flip->var_cnt; i++) 
-    buf_copy_str_rpad (flip->output_buf[i].s, MAX_SHORT_STRING,
-                       var_get_name (flip->var[i]));
-  if (fwrite (flip->output_buf, sizeof *flip->output_buf,
-              flip->var_cnt, flip->file) != (size_t) flip->var_cnt) 
-    {
-      msg (SE, _("Error writing FLIP file: %s."), strerror (errno));
-      return NULL;
-    }
-
-  flip->case_cnt = 1;
-
-  return create_case_sink (&flip_sink_class,
-			   dataset_dict (ds),
-			   dataset_get_casefile_factory (ds),
-			   flip);
+  return true;
 }
 
 /* Writes case C to the FLIP sink.
    Returns true if successful, false if an I/O error occurred. */
 static bool
-flip_sink_write (struct case_sink *sink, const struct ccase *c)
+write_flip_case (struct flip_pgm *flip, const struct ccase *c)
 {
-  struct flip_pgm *flip = sink->aux;
   size_t i;
-  
+
   flip->case_cnt++;
 
   if (flip->new_names != NULL)
     {
       struct varname *v = pool_alloc (flip->pool, sizeof *v);
-      int fv = flip->idx_to_fv[var_get_dict_index (flip->new_names)];
       v->next = NULL;
       if (var_is_numeric (flip->new_names))
         {
-          double f = case_num_idx (c, fv);
+          double f = case_num (c, flip->new_names);
 
           if (f == SYSMIS)
             strcpy (v->name, "VSYSMIS");
@@ -354,10 +331,10 @@ flip_sink_write (struct case_sink *sink, const struct ccase *c)
       else
 	{
 	  int width = MIN (var_get_width (flip->new_names), MAX_SHORT_STRING);
-	  memcpy (v->name, case_str_idx (c, fv), width);
+	  memcpy (v->name, case_str (c, flip->new_names), width);
 	  v->name[width] = 0;
 	}
-      
+
       if (flip->new_names_head == NULL)
 	flip->new_names_head = v;
       else
@@ -368,23 +345,9 @@ flip_sink_write (struct case_sink *sink, const struct ccase *c)
   /* Write to external file. */
   for (i = 0; i < flip->var_cnt; i++)
     {
-      double out;
-      
-      if (var_is_numeric (flip->var[i])) 
-        {
-          int fv = flip->idx_to_fv[var_get_dict_index (flip->var[i])];
-          out = case_num_idx (c, fv); 
-        }
-      else
-        out = SYSMIS;
-      flip->output_buf[i].f = out;
-    }
-	  
-  if (fwrite (flip->output_buf, sizeof *flip->output_buf,
-              flip->var_cnt, flip->file) != (size_t) flip->var_cnt) 
-    {
-      msg (SE, _("Error writing FLIP file: %s."), strerror (errno));
-      return false; 
+      const struct variable *v = flip->var[i];
+      double out = var_is_numeric (v) ? case_num (c, v) : SYSMIS;
+      fwrite (&out, sizeof out, 1, flip->file);
     }
   return true;
 }
@@ -428,38 +391,38 @@ flip_file (struct flip_pgm *flip)
   output_buf = input_buf + flip->var_cnt * case_capacity;
 
   input_file = flip->file;
-  if (fseek (input_file, 0, SEEK_SET) != 0) 
+  if (fseek (input_file, 0, SEEK_SET) != 0)
     {
       msg (SE, _("Error rewinding FLIP file: %s."), strerror (errno));
       return false;
     }
-      
+
   output_file = pool_tmpfile (flip->pool);
-  if (output_file == NULL) 
+  if (output_file == NULL)
     {
       msg (SE, _("Error creating FLIP source file."));
       return false;
     }
-  
+
   for (case_idx = 0; case_idx < flip->case_cnt; )
     {
       unsigned long read_cases = MIN (flip->case_cnt - case_idx,
                                       case_capacity);
       size_t i;
 
-      if (read_cases != fread (input_buf, case_bytes, read_cases, input_file)) 
+      if (read_cases != fread (input_buf, case_bytes, read_cases, input_file))
         {
           if (ferror (input_file))
             msg (SE, _("Error reading FLIP file: %s."), strerror (errno));
           else
-            msg (SE, _("Unexpected end of file reading FLIP file.")); 
+            msg (SE, _("Unexpected end of file reading FLIP file."));
           return false;
         }
 
       for (i = 0; i < flip->var_cnt; i++)
 	{
 	  unsigned long j;
-	  
+
 	  for (j = 0; j < read_cases; j++)
 	    output_buf[j] = input_buf[i + j * flip->var_cnt];
 
@@ -474,7 +437,7 @@ flip_file (struct flip_pgm *flip)
 	  if (fseeko (output_file,
                       sizeof *input_buf * (case_idx
                                            + (off_t) i * flip->case_cnt),
-                      SEEK_SET) != 0) 
+                      SEEK_SET) != 0)
             {
               msg (SE, _("Error seeking FLIP source file: %s."),
                    strerror (errno));
@@ -482,11 +445,11 @@ flip_file (struct flip_pgm *flip)
             }
 
 	  if (fwrite (output_buf, sizeof *output_buf, read_cases, output_file)
-	      != read_cases) 
+	      != read_cases)
             {
               msg (SE, _("Error writing FLIP source file: %s."),
                    strerror (errno));
-              return false; 
+              return false;
             }
 	}
 
@@ -500,67 +463,49 @@ flip_file (struct flip_pgm *flip)
     }
   pool_unregister (flip->pool, input_buf);
   free (input_buf);
-  
-  if (fseek (output_file, 0, SEEK_SET) != 0) 
+
+  if (fseek (output_file, 0, SEEK_SET) != 0)
     {
       msg (SE, _("Error rewinding FLIP source file: %s."), strerror (errno));
-      return false; 
+      return false;
     }
   flip->file = output_file;
 
   return true;
 }
 
-/* FLIP sink class. */
-static const struct case_sink_class flip_sink_class = 
-  {
-    "FLIP",
-    NULL,
-    flip_sink_write,
-    NULL,
-    NULL,
-  };
-
-/* Creates and returns a FLIP source based on PGM,
-   which should have already been used as a sink. */
-static struct case_source *
-flip_source_create (struct flip_pgm *pgm)
-{
-  return create_case_source (&flip_source_class, pgm);
-}
-
 /* Reads one case into C.
    Returns true if successful, false at end of file or if an
    I/O error occurred. */
 static bool
-flip_source_read (struct case_source *source, struct ccase *c)
+flip_casereader_read (struct casereader *reader UNUSED, void *flip_,
+                      struct ccase *c)
 {
-  struct flip_pgm *flip = source->aux;
+  struct flip_pgm *flip = flip_;
   size_t i;
 
   if (flip->error || flip->cases_read >= flip->var_cnt)
     return false;
-  
-  if (flip->input_buf == NULL)
-    flip->input_buf = pool_nmalloc (flip->pool,
-                                    flip->case_cnt, sizeof *flip->input_buf);
-
-  if (fread (flip->input_buf, sizeof *flip->input_buf, flip->case_cnt,
-             flip->file) != flip->case_cnt) 
-    {
-      if (ferror (flip->file))
-        msg (SE, _("Error reading FLIP temporary file: %s."),
-             strerror (errno));
-      else if (feof (flip->file))
-        msg (SE, _("Unexpected end of file reading FLIP temporary file."));
-      else
-        NOT_REACHED ();
-      flip->error = true;
-      return false;
-    }
 
+  case_create (c, flip->case_cnt);
   for (i = 0; i < flip->case_cnt; i++)
-    case_data_rw_idx (c, i)->f = flip->input_buf[i].f;
+    {
+      double in;
+      if (fread (&in, sizeof in, 1, flip->file) != 1)
+        {
+          case_destroy (c);
+          if (ferror (flip->file))
+            msg (SE, _("Error reading FLIP temporary file: %s."),
+                 strerror (errno));
+          else if (feof (flip->file))
+            msg (SE, _("Unexpected end of file reading FLIP temporary file."));
+          else
+            NOT_REACHED ();
+          flip->error = true;
+          return false;
+        }
+      case_data_rw_idx (c, i)->f = in;
+    }
 
   flip->cases_read++;
 
@@ -570,19 +515,19 @@ flip_source_read (struct case_source *source, struct ccase *c)
 /* Destroys the source.
    Returns true if successful read, false if an I/O occurred
    during destruction or previously. */
-static bool
-flip_source_destroy (struct case_source *source)
+static void
+flip_casereader_destroy (struct casereader *reader UNUSED, void *flip_)
 {
-  struct flip_pgm *flip = source->aux;
-  bool ok = !flip->error;
+  struct flip_pgm *flip = flip_;
+  if (flip->error)
+    casereader_force_error (reader);
   destroy_flip_pgm (flip);
-  return ok;
 }
 
-static const struct case_source_class flip_source_class = 
+static const struct casereader_class flip_casereader_class =
   {
-    "FLIP",
+    flip_casereader_read,
+    flip_casereader_destroy,
+    NULL,
     NULL,
-    flip_source_read,
-    flip_source_destroy
   };
