@@ -1,20 +1,18 @@
-/* PSPP - computes sample statistics.
+/* PSPP - a program for statistical analysis.
    Copyright (C) 1997-9, 2000 Free Software Foundation, Inc.
 
-   This program is free software; you can redistribute it and/or
-   modify it under the terms of the GNU General Public License as
-   published by the Free Software Foundation; either version 2 of the
-   License, or (at your option) any later version.
+   This program is free software: you can redistribute it and/or modify
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation, either version 3 of the License, or
+   (at your option) any later version.
 
-   This program is distributed in the hope that it will be useful, but
-   WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   General Public License for more details.
+   This program is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+   GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
-   02110-1301, USA. */
+   along with this program.  If not, see <http://www.gnu.org/licenses/>. */
 
 #include <config.h>
 
@@ -48,7 +46,7 @@
 #include <libpspp/getl.h>
 
 
-struct syntax_file_source 
+struct syntax_file_source
   {
     struct getl_interface parent ;
 
@@ -76,9 +74,9 @@ line_number (const struct getl_interface *s)
 
 /* Reads a line from syntax file source S into LINE.
    Returns true if successful, false at end of file. */
-bool
+static bool
 read_syntax_file (struct getl_interface *s,
-                  struct string *line, enum getl_syntax *syntax)
+                  struct string *line)
 {
   struct syntax_file_source *sfs = (struct syntax_file_source *) s;
 
@@ -97,7 +95,7 @@ read_syntax_file (struct getl_interface *s,
 
   /* Read line from file and remove new-line.
      Skip initial "#! /usr/bin/pspp" line. */
-  do 
+  do
     {
       sfs->ln++;
       if (!ds_read_line (line, sfs->syntax_file))
@@ -114,7 +112,6 @@ read_syntax_file (struct getl_interface *s,
   if (get_echo ())
     tab_output_text (TAB_LEFT | TAB_FIX, ds_cstr (line));
 
-  *syntax = GETL_BATCH;
   return true;
 }
 
@@ -138,7 +135,7 @@ always_false (const struct getl_interface *s UNUSED)
 
 /* Creates a syntax file source with file name FN. */
 struct getl_interface *
-create_syntax_file_source (const char *fn) 
+create_syntax_file_source (const char *fn)
 {
   struct syntax_file_source *ss = xzalloc (sizeof (*ss));
 
