@@ -1,20 +1,18 @@
-/* PSPP - computes sample statistics.
+/* PSPP - a program for statistical analysis.
    Copyright (C) 2004 Free Software Foundation, Inc.
 
-   This program is free software; you can redistribute it and/or
-   modify it under the terms of the GNU General Public License as
-   published by the Free Software Foundation; either version 2 of the
-   License, or (at your option) any later version.
+   This program is free software: you can redistribute it and/or modify
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation, either version 3 of the License, or
+   (at your option) any later version.
 
-   This program is distributed in the hope that it will be useful, but
-   WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   General Public License for more details.
+   This program is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+   GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
-   02110-1301, USA. */
+   along with this program.  If not, see <http://www.gnu.org/licenses/>. */
 
 #include <config.h>
 #include <stdio.h>
@@ -49,10 +47,10 @@ struct chart {
   char *file_name;
   FILE *file;
 
-  /* The geometry of the chart 
+  /* The geometry of the chart
      See diagram at the foot of this file.
    */
-  
+
   int data_top   ;
   int data_right ;
   int data_bottom;
@@ -67,9 +65,9 @@ struct chart {
   int legend_left ;
   int legend_right ;
 
-  
+
   /* Default font size for the plot (if zero, then use plotter default) */
-  int font_size; 
+  int font_size;
 
   char fill_colour[10];
 
@@ -87,4 +85,11 @@ struct chart {
 struct chart * chart_create(void);
 void chart_submit(struct chart *ch);
 
+/* Helper functions for output drivers that put each chart into a
+   separate file. */
+void chart_init_separate (struct chart *, const char *type,
+                          const char *file_name_tmpl, int number);
+
+void chart_finalise_separate (struct chart *);
+
 #endif
