@@ -1,20 +1,18 @@
-/* PSPP - computes sample statistics.
-   Copyright (C) 1997-9, 2000, 2006 Free Software Foundation, Inc.
+/* PSPP - a program for statistical analysis.
+   Copyright (C) 1997-9, 2000, 2006, 2007 Free Software Foundation, Inc.
 
-   This program is free software; you can redistribute it and/or
-   modify it under the terms of the GNU General Public License as
-   published by the Free Software Foundation; either version 2 of the
-   License, or (at your option) any later version.
+   This program is free software: you can redistribute it and/or modify
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation, either version 3 of the License, or
+   (at your option) any later version.
 
-   This program is distributed in the hope that it will be useful, but
-   WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   General Public License for more details.
+   This program is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+   GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
-   02110-1301, USA. */
+   along with this program.  If not, see <http://www.gnu.org/licenses/>. */
 
 #include <config.h>
 
@@ -40,7 +38,6 @@
 #include "afm.h"
 #include "chart.h"
 #include "error.h"
-#include "getline.h"
 #include "intprops.h"
 #include "manager.h"
 #include "minmax.h"
@@ -141,8 +138,7 @@ ps_open_driver (struct outp_driver *this, struct substring options)
   x->draw_headers = true;
   x->page_number = 0;
   x->portrait = true;
-  x->paper_width = PSUS * 17 / 2;
-  x->paper_length = PSUS * 11;
+  outp_get_paper_size ("", &x->paper_width, &x->paper_length);
   x->left_margin = PSUS / 2;
   x->right_margin = PSUS / 2;
   x->top_margin = PSUS / 2;
@@ -163,7 +159,7 @@ ps_open_driver (struct outp_driver *this, struct substring options)
       goto error;
     }
 
-  if (x->portrait) 
+  if (x->portrait)
     {
       this->width = x->paper_width;
       this->length = x->paper_length;
@@ -172,7 +168,7 @@ ps_open_driver (struct outp_driver *this, struct substring options)
     {
       this->width = x->paper_length;
       this->length = x->paper_width;
-    }    
+    }
   this->width -= x->left_margin + x->right_margin;
   this->length -= x->top_margin + x->bottom_margin;
   if (x->draw_headers)
@@ -233,18 +229,23 @@ static bool
 ps_close_driver (struct outp_driver *this)
 {
   struct ps_driver_ext *x = this->ext;
-  bool ok;
+  bool ok = true;
   size_t i;
 
-  fprintf (x->file,
-	   "%%%%Trailer\n"
-           "%%%%Pages: %d\n"
-           "%%%%EOF\n",
-           x->page_number);
+  if (x->file != NULL)
+    {
+      fprintf (x->file,
+               "%%%%Trailer\n"
+               "%%%%Pages: %d\n"
+               "%%%%EOF\n",
+               x->page_number);
+
+      ok = fn_close (x->file_name, x->file) == 0;
+      if (!ok)
+        error (0, errno, _("closing PostScript output file \"%s\""),
+               x->file_name);
+    }
 
-  ok = fn_close (x->file_name, x->file) == 0;
-  if (!ok)
-    error (0, errno, _("closing PostScript output file \"%s\""), x->file_name);
   free (x->file_name);
   for (i = 0; i < OUTP_FONT_CNT; i++)
     free_font (x->fonts[i]);
@@ -378,14 +379,10 @@ handle_option (struct outp_driver *this, const char *key,
       break;
     case dimension_arg:
       {
-	int dimension = outp_evaluate_dimension (value, NULL);
+	int dimension = outp_evaluate_dimension (value);
 
 	if (dimension <= 0)
-	  {
-	    error (0, 0, _("value for `%s' must be a dimension of positive "
-                           "length (i.e., `1in')"), key);
-	    break;
-	  }
+          break;
 	switch (subcat)
 	  {
 	  case 0:
@@ -796,7 +793,7 @@ draw_text (struct outp_driver *this,
 static void
 draw_header_line (struct outp_driver *this,
                   const char *left, const char *right,
-                  int x0, int x1, int y) 
+                  int x0, int x1, int y)
 {
   int right_width = 0;
   if (right != NULL)
@@ -827,12 +824,12 @@ draw_headers (struct outp_driver *this)
 
   r1 = xasprintf (_("%s - Page %d"), get_start_date (), ext->page_number);
   r2 = xasprintf ("%s - %s", version, host_system);
- 
+
   draw_header_line (this, outp_title, r1, x0, x1, y);
   y += this->font_height;
-  
+
   draw_header_line (this, outp_subtitle, r2, x0, x1, y);
- 
+
   free (r1);
   free (r2);
 }
@@ -847,7 +844,7 @@ draw_headers (struct outp_driver *this)
 static void
 write_text (struct outp_driver *this,
             int x0, int y0,
-            enum outp_font font, 
+            enum outp_font font,
             enum outp_justification justification,
             const struct afm_character **chars, int *kerns, size_t char_cnt,
             int width_left)
@@ -1014,7 +1011,7 @@ text (struct outp_driver *this, const struct outp_text *text, bool draw,
       char_width = cur->width * this->font_height / 1000;
 
       /* Get kern adjustment. */
-      if (s.glyph_cnt > 0) 
+      if (s.glyph_cnt > 0)
         kern_adjust = (afm_get_kern_adjustment (s.glyphs[s.glyph_cnt - 1], cur)
                        * this->font_height / 1000);
       else
@@ -1435,6 +1432,7 @@ const struct outp_class postscript_class =
 
   ps_open_page,
   ps_close_page,
+  NULL,
 
   ps_submit,
 
