@@ -1,5 +1,5 @@
 /* PSPPIRE - a graphical user interface for PSPP.
-   Copyright (C) 2007, 2010, 2011, 2012, 2013  Free Software Foundation
+   Copyright (C) 2007  Free Software Foundation
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -16,19 +16,21 @@
 
 #include <config.h>
 
+#include <gtksheet/gtksheet.h>
 #include "psppire-dialog.h"
-#include "builder-wrapper.h"
-#include "psppire-data-window.h"
-#include "psppire-data-editor.h"
-#include "executor.h"
 #include "helper.h"
-#include <ui/syntax-gen.h>
+#include "data-editor.h"
+#include <language/syntax-string-source.h>
+#include "syntax-editor.h"
+#include "psppire-var-store.h"
+#include <libpspp/syntax-gen.h>
 
 #include "comments-dialog.h"
 
 #include "dialog-common.h"
 
 #include <gtk/gtk.h>
+#include <glade/glade.h>
 
 #include <gettext.h>
 
@@ -38,7 +40,7 @@
 
 struct comment_dialog
 {
-  GtkBuilder *xml;
+  GladeXML *xml;
   PsppireDict *dict;
 };
 
@@ -74,31 +76,42 @@ wrap_line (GtkTextBuffer *buffer,
   if ( chars > DOC_LINE_LENGTH )
     {
       GtkTextIter line_fold = *iter;
+      GtkTextIter cursor;
 
       gtk_text_iter_set_line_offset (&line_fold, DOC_LINE_LENGTH);
 
       gtk_text_buffer_insert (buffer, &line_fold, "\r\n", 2);
+
+      cursor = line_fold;
+      gtk_text_iter_forward_to_line_end (&cursor);
+
+      gtk_text_buffer_place_cursor (buffer, &cursor);
     }
 
 }
 
 
 void
-comments_dialog (PsppireDataWindow *de)
+comments_dialog (GObject *o, gpointer data)
 {
   GtkTextIter iter;
   gint response ;
+  struct data_editor *de = data;
   struct comment_dialog cd;
 
-  GtkBuilder *xml = builder_new ("psppire.ui");
+  GladeXML *xml = XML_NEW ("psppire.glade");
 
   GtkWidget *dialog = get_widget_assert (xml, "comments-dialog");
   GtkWidget *textview = get_widget_assert (xml, "comments-textview1");
   GtkWidget *label = get_widget_assert (xml, "column-number-label");
   GtkTextBuffer *buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (textview));
 
+  GtkSheet *var_sheet =
+    GTK_SHEET (get_widget_assert (de->xml, "variable_sheet"));
 
-  gtk_window_set_transient_for (GTK_WINDOW (dialog), GTK_WINDOW (de));
+  PsppireVarStore *vs = PSPPIRE_VAR_STORE (gtk_sheet_get_model (var_sheet));
+
+  gtk_window_set_transient_for (GTK_WINDOW (dialog), de->parent.window);
 
   {
     PangoContext * context ;
@@ -134,7 +147,7 @@ comments_dialog (PsppireDataWindow *de)
   }
 
   cd.xml = xml;
-  g_object_get (de->data_editor, "dictionary", &cd.dict, NULL);
+  cd.dict = vs->dict;
 
   g_signal_connect (buffer, "mark-set",
 		    G_CALLBACK (set_column_number), label);
@@ -154,10 +167,25 @@ comments_dialog (PsppireDataWindow *de)
   switch (response)
     {
     case GTK_RESPONSE_OK:
-      g_free (execute_syntax_string (de, generate_syntax (&cd)));
+      {
+	gchar *syntax = generate_syntax (&cd);
+	struct getl_interface *sss = create_syntax_string_source (syntax);
+	execute_syntax (sss);
+
+	g_free (syntax);
+      }
       break;
     case PSPPIRE_RESPONSE_PASTE:
-      g_free (paste_syntax_to_window (generate_syntax (&cd)));
+      {
+	gchar *syntax = generate_syntax (&cd);
+
+	struct syntax_editor *se =
+	  (struct syntax_editor *) window_create (WINDOW_SYNTAX, NULL);
+
+	gtk_text_buffer_insert_at_cursor (se->buffer, syntax, -1);
+
+	g_free (syntax);
+      }
       break;
     default:
       break;
@@ -186,7 +214,13 @@ refresh (PsppireDialog *dialog, const struct comment_dialog *cd)
   gtk_text_buffer_set_text (buffer, "", 0);
 
   for ( i = 0 ; i < dict_get_document_line_cnt (cd->dict->dict); ++i )
-    add_line_to_buffer (buffer, dict_get_document_line (cd->dict->dict, i));
+    {
+      struct string str;
+      ds_init_empty (&str);
+      dict_get_document_line (cd->dict->dict, i, &str);
+      add_line_to_buffer (buffer, ds_cstr (&str));
+      ds_destroy (&str);
+    }
 }
 
 
@@ -201,37 +235,38 @@ generate_syntax (const struct comment_dialog *cd)
   GtkWidget *tv = get_widget_assert (cd->xml, "comments-textview1");
   GtkWidget *check = get_widget_assert (cd->xml, "comments-checkbutton1");
   GtkTextBuffer *buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (tv));
+  const char *existing_docs = dict_get_documents (cd->dict->dict);
 
   str = g_string_new ("\n* Data File Comments.\n\n");
 
-  if (dict_get_documents (cd->dict->dict) != NULL)
+  if ( NULL != existing_docs)
     g_string_append (str, "DROP DOCUMENTS.\n");
 
   g_string_append (str, "ADD DOCUMENT\n");
 
   for (i = 0 ; i < gtk_text_buffer_get_line_count (buffer) ; ++i )
     {
-      struct string tmp;
-      GtkTextIter start;
-      char *line;
-
+      struct string line;
+      GtkTextIter start, end;
       gtk_text_buffer_get_iter_at_line (buffer, &start, i);
-      if (gtk_text_iter_ends_line (&start))
-	line = g_strdup ("");
+
+      end = start;
+
+      gtk_text_iter_forward_to_line_end (&end);
+
+      if ( gtk_text_iter_ends_line (&start))
+	ds_init_cstr (&line, "");
       else
-        {
-          GtkTextIter end = start;
-          gtk_text_iter_forward_to_line_end (&end);
-          line = gtk_text_buffer_get_text (buffer, &start, &end, FALSE);
-        }
+	ds_init_cstr (&line,
+		      gtk_text_buffer_get_text (buffer,
+						&start, &end,
+						FALSE));
 
-      ds_init_empty (&tmp);
-      syntax_gen_string (&tmp, ss_cstr (line));
-      g_free (line);
+      gen_quoted_string (&line);
 
-      g_string_append_printf (str, " %s\n", ds_cstr (&tmp));
+      g_string_append_printf (str, " %s\n", ds_cstr (&line));
 
-      ds_destroy (&tmp);
+      ds_destroy (&line);
     }
   g_string_append (str, " .\n");
 
