@@ -1,5 +1,5 @@
 /* PSPPIRE - a graphical user interface for PSPP.
-   Copyright (C) 2007, 2010  Free Software Foundation
+   Copyright (C) 2007  Free Software Foundation
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -17,8 +17,11 @@
 #ifndef __COMPUTE_DIALOG_H
 #define __COMPUTE_DIALOG_H
 
-#include "psppire-data-window.h"
 
-void compute_dialog (PsppireDataWindow * data);
+#include <gtk/gtk.h>
+#include <glade/glade.h>
+
+
+void compute_dialog (GObject *o, gpointer data);
 
 #endif
