@@ -1,5 +1,5 @@
 /* PSPPIRE - a graphical user interface for PSPP.
-   Copyright (C) 2007, 2009, 2011, 2012, 2015, 2020  Free Software Foundation
+   Copyright (C) 2007  Free Software Foundation
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -20,42 +20,44 @@ which match particular strings */
 
 #include <config.h>
 
+#include "find-dialog.h"
+#include "psppire-selector.h"
+#include "psppire-dialog.h"
+#include "helper.h"
+#include "data-editor.h"
+#include "dict-display.h"
+#include <data/value.h>
+#include <data/datasheet.h>
+#include <data/data-in.h>
+#include "psppire-data-store.h"
+#include <libpspp/alloc.h>
 #include <ctype.h>
-#include <gtk/gtk.h>
-#include <regex.h>
-#include <stdlib.h>
 #include <sys/types.h>
-#include <math.h>
+#include <regex.h>
+#include <libpspp/message.h>
 
-#include "data/data-in.h"
-#include "data/datasheet.h"
-#include "data/format.h"
-#include "data/value.h"
-#include "libpspp/cast.h"
-#include "libpspp/message.h"
-#include "ui/gui/builder-wrapper.h"
-#include "ui/gui/dict-display.h"
-#include "ui/gui/find-dialog.h"
-#include "ui/gui/helper.h"
-#include "ui/gui/psppire-data-store.h"
-#include "ui/gui/psppire-data-window.h"
-#include "ui/gui/psppire-dialog.h"
-#include "ui/gui/psppire-selector.h"
-#include <ssw-sheet.h>
 
-#include "gl/xalloc.h"
+
+#include <gtk/gtk.h>
+#include <glade/glade.h>
+#include <stdlib.h>
 
 #include <gettext.h>
 #define _(msgid) gettext (msgid)
 #define N_(msgid) msgid
 
 
+/* FIXME: These shouldn't be here */
+#include <gtksheet/gtksheet.h>
+#include "psppire-var-store.h"
+
 struct find_dialog
 {
-  GtkBuilder *xml;
+  GladeXML *xml;
   PsppireDict *dict;
+  GtkSheet *data_sheet;
   struct datasheet *data;
-  PsppireDataWindow *de;
+  struct data_editor *de;
   GtkWidget *variable_entry;
   GtkWidget *value_entry;
   GtkWidget *value_labels_checkbox;
@@ -65,7 +67,7 @@ struct find_dialog
 
 static void
 find_value (const struct find_dialog *fd, casenumber current_row,
-           casenumber *row, int *column);
+	   casenumber *row, int *column);
 
 
 /* A callback which occurs whenever the "Refresh" button is clicked,
@@ -87,7 +89,7 @@ refresh (GObject *obj, const struct find_dialog *fd)
   gtk_entry_set_text (GTK_ENTRY (fd->value_entry), "");
 
   gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (fd->match_regexp_checkbox),
-                                FALSE);
+				FALSE);
 
   gtk_toggle_button_set_active
     (GTK_TOGGLE_BUTTON (fd->match_substring_checkbox), FALSE);
@@ -101,22 +103,23 @@ refresh (GObject *obj, const struct find_dialog *fd)
 static void
 do_find (GObject *obj, const struct find_dialog *fd)
 {
-  SswSheet *sheet = SSW_SHEET (fd->de->data_editor->data_sheet);
   casenumber x = -1;
   gint column = -1;
-  gint unused;
-  gint row = 0;
-  ssw_sheet_get_active_cell (sheet, &unused, &row);
+  gint row;
+  gtk_sheet_get_active_cell (fd->data_sheet, &row, NULL);
+
+  if ( row < 0 )
+    row = 0;
 
   find_value (fd, row, &x, &column);
 
-  if (x != -1)
+  if ( x != -1)
     {
-      gtk_notebook_set_current_page (GTK_NOTEBOOK (fd->de->data_editor),
-                                     PSPPIRE_DATA_EDITOR_DATA_VIEW);
+      data_editor_select_sheet (fd->de, PAGE_DATA_SHEET);
 
-      ssw_sheet_scroll_to (sheet, column, x);
-      ssw_sheet_set_active_cell (sheet, column, x, NULL);
+      gtk_sheet_moveto (fd->data_sheet, x, column, 0.5, 0.5);
+
+      gtk_sheet_set_active_cell (fd->data_sheet, x, column);
     }
 }
 
@@ -133,17 +136,17 @@ on_select (GtkEntry *entry, gpointer data)
   g_return_if_fail (var);
 
   gtk_widget_set_sensitive (fd->value_labels_checkbox,
-                            var_has_value_labels (var));
+			    var_has_value_labels (var));
 
   search_labels =
     gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (fd->value_labels_checkbox));
 
   gtk_widget_set_sensitive (fd->match_regexp_checkbox,
-                            var_is_alpha (var) || search_labels);
+			    var_is_alpha (var) || search_labels);
 
 
   gtk_widget_set_sensitive (fd->match_substring_checkbox,
-                            var_is_alpha (var) || search_labels);
+			    var_is_alpha (var) || search_labels);
 }
 
 /* Callback on the selector.
@@ -169,17 +172,19 @@ value_labels_toggled (GtkToggleButton *tb, gpointer data)
   gboolean active = gtk_toggle_button_get_active  (tb) ;
 
   gtk_widget_set_sensitive (fd->match_substring_checkbox,
-                            active || (var && var_is_alpha (var)));
+			    active || (var && var_is_alpha (var)));
 
   gtk_widget_set_sensitive (fd->match_regexp_checkbox,
-                              active || (var && var_is_alpha (var)));
+			      active || (var && var_is_alpha (var)));
 }
 
 /* Pops up the Find dialog box
  */
 void
-find_dialog (PsppireDataWindow *de)
+find_dialog (GObject *o, gpointer data)
 {
+  struct data_editor *de = data;
+
   struct find_dialog fd;
 
   GtkWidget *dialog ;
@@ -187,71 +192,81 @@ find_dialog (PsppireDataWindow *de)
   GtkWidget *selector;
   GtkWidget *find_button;
 
+  GtkSheet *var_sheet;
+  GtkSheet *data_sheet ;
   GtkWidget *buttonbox;
 
+  PsppireVarStore *vs  ;
   PsppireDataStore *ds ;
 
-  fd.xml = builder_new ("find.ui");
+  fd.xml = XML_NEW ("psppire.glade");
   fd.de = de;
 
-  find_button = gtk_button_new_with_label (_("Find"));
+  find_button = gtk_button_new_from_stock  (GTK_STOCK_FIND);
   gtk_widget_show (find_button);
 
   buttonbox = get_widget_assert (fd.xml, "find-buttonbox");
 
-  psppire_box_pack_start_defaults (GTK_BOX (buttonbox), find_button);
+  gtk_box_pack_start_defaults (GTK_BOX (buttonbox), find_button);
   gtk_box_reorder_child (GTK_BOX (buttonbox), find_button, 0);
 
   dialog = get_widget_assert (fd.xml, "find-dialog");
   source = get_widget_assert (fd.xml, "find-variable-treeview");
   selector = get_widget_assert (fd.xml, "find-selector");
 
-  g_object_get (de->data_editor,
-                "dictionary", &fd.dict,
-                "data-store", &ds,
-                NULL);
+  var_sheet = GTK_SHEET (get_widget_assert (de->xml, "variable_sheet"));
+  data_sheet = GTK_SHEET (get_widget_assert (de->xml, "data_sheet"));
 
-  fd.data = ds->datasheet;
+  vs = PSPPIRE_VAR_STORE (gtk_sheet_get_model (var_sheet));
+  ds = PSPPIRE_DATA_STORE (gtk_sheet_get_model (data_sheet));
+
+  fd.dict = vs->dict;
+  fd.data = ds->case_file->datasheet;
+  fd.data_sheet = data_sheet;
 
   fd.variable_entry        = get_widget_assert (fd.xml, "find-variable-entry");
   fd.value_entry           = get_widget_assert (fd.xml, "find-value-entry");
   fd.value_labels_checkbox =
     get_widget_assert (fd.xml,
-                       "find-value-labels-checkbutton");
+		       "find-value-labels-checkbutton");
 
   fd.match_regexp_checkbox =
     get_widget_assert (fd.xml,
-                       "find-match-regexp-checkbutton");
+		       "find-match-regexp-checkbutton");
 
   fd.match_substring_checkbox =
     get_widget_assert (fd.xml,
-                       "find-match-substring-checkbutton");
+		       "find-match-substring-checkbutton");
 
 
 
-  gtk_window_set_transient_for (GTK_WINDOW (dialog), GTK_WINDOW (de));
+  gtk_window_set_transient_for (GTK_WINDOW (dialog), de->parent.window);
 
 
-  g_object_set (source, "model", fd.dict,
-        "selection-mode", GTK_SELECTION_SINGLE,
-        NULL);
+  attach_dictionary_to_treeview (GTK_TREE_VIEW (source),
+				 fd.dict,
+				 GTK_SELECTION_SINGLE,
+				 NULL);
 
-
-  psppire_selector_set_filter_func (PSPPIRE_SELECTOR (selector),
-                                    is_currently_in_entry);
+  psppire_selector_set_subjects (PSPPIRE_SELECTOR (selector),
+				 source,
+				 fd.variable_entry,
+				 insert_source_row_into_entry,
+				 is_currently_in_entry
+				 );
 
   g_signal_connect (dialog, "refresh", G_CALLBACK (refresh),  &fd);
 
   g_signal_connect (find_button, "clicked", G_CALLBACK (do_find),  &fd);
 
   g_signal_connect (selector, "selected",
-                    G_CALLBACK (on_select),  &fd);
+		    G_CALLBACK (on_select),  &fd);
 
   g_signal_connect (selector, "de-selected",
-                    G_CALLBACK (on_deselect),  &fd);
+		    G_CALLBACK (on_deselect),  &fd);
 
   g_signal_connect (fd.value_labels_checkbox, "toggled",
-                    G_CALLBACK (value_labels_toggled),  &fd);
+		    G_CALLBACK (value_labels_toggled),  &fd);
 
 
   psppire_dialog_run (PSPPIRE_DIALOG (dialog));
@@ -272,7 +287,7 @@ forward (casenumber *i, struct datasheet *data UNUSED)
 static void
 forward_wrap (casenumber *i, struct datasheet *data)
 {
-  if (++*i >=  datasheet_get_n_rows (data)) *i = 0;
+  if ( ++*i >=  datasheet_get_row_cnt (data) ) *i = 0;
 }
 
 static void
@@ -285,8 +300,8 @@ backward (casenumber *i, struct datasheet *data UNUSED)
 static void
 backward_wrap (casenumber *i, struct datasheet *data)
 {
-  if (--*i < 0)
-    *i = datasheet_get_n_rows (data) - 1;
+  if ( --*i < 0 )
+    *i = datasheet_get_row_cnt (data) - 1;
 }
 
 
@@ -313,9 +328,6 @@ cp1c (casenumber current, struct datasheet *data)
 static casenumber
 cm1 (casenumber current, struct datasheet *data)
 {
-  if (current == 0)
-    return datasheet_get_n_rows (data) - 1;
-
   return current - 1;
 }
 
@@ -334,7 +346,7 @@ cm1c (casenumber current, struct datasheet *data)
 static casenumber
 last (casenumber current, struct datasheet *data)
 {
-  return datasheet_get_n_rows (data) ;
+  return datasheet_get_row_cnt (data) ;
 }
 
 static casenumber
@@ -343,7 +355,7 @@ minus1 (casenumber current, struct datasheet *data)
   return -1;
 }
 
-/* A type to facilitate iterating through casenumbers */
+/* An type to facilitate iterating through casenumbers */
 struct casenum_iterator
 {
   /* returns the first case to access */
@@ -364,25 +376,18 @@ enum iteration_type{
   n_iterators
 };
 
-static const struct casenum_iterator case_iterator[n_iterators] =
+static struct casenum_iterator ip[n_iterators] =
   {
-   /* Forward iterator (linear) */
     {cp1, last, forward},
-
-    /* Forward iterator (circular) */
     {cp1c, cm1, forward_wrap},
-
-    /* Reverse iterator (linear) */
     {cm1, minus1, backward},
-
-    /* Reverse iterator (circular */
     {cm1c, cp1, backward_wrap}
   };
 
 
 
 /* A factory returning an iterator according to the dialog box's settings */
-static const struct casenum_iterator *
+static struct casenum_iterator *
 get_iteration_params (const struct find_dialog *fd)
 {
   gboolean wrap = gtk_toggle_button_get_active
@@ -391,19 +396,19 @@ get_iteration_params (const struct find_dialog *fd)
   gboolean reverse = gtk_toggle_button_get_active
     (GTK_TOGGLE_BUTTON (get_widget_assert (fd->xml, "find-backwards")));
 
-  if (wrap)
+  if ( wrap )
     {
-      if (reverse)
-        return &case_iterator[REVERSE_WRAP];
+      if ( reverse )
+	return &ip[REVERSE_WRAP];
       else
-        return &case_iterator[FORWARD_WRAP];
+	return &ip[FORWARD_WRAP];
     }
   else
     {
-      if (reverse)
-        return &case_iterator[REVERSE];
+      if ( reverse )
+	return &ip[REVERSE];
       else
-        return &case_iterator[FORWARD];
+	return &ip[FORWARD];
     }
 }
 
@@ -411,11 +416,11 @@ get_iteration_params (const struct find_dialog *fd)
 enum string_cmp_flags
   {
     STR_CMP_SUBSTR = 0x01, /* Find strings which are substrings of the
-                              values */
+			      values */
     STR_CMP_REGEXP = 0x02, /* Match against a regular expression */
 
     STR_CMP_LABELS = 0x04  /* Match against the values' labels instead
-                              of the data */
+			      of the data */
   };
 
 
@@ -426,19 +431,17 @@ struct comparator
   enum string_cmp_flags flags;
 
   bool (*compare) (const struct comparator *,
-                   const union value *);
+		   const union value *);
 
   void (*destroy) (struct comparator *);
 };
 
 
-/* A comparator which operates on the numerical values,
-   rounded to the number of decimal places indicated by
-   the variable's format.  */
-struct numeric_comparator
+/* A comparator which operates on the unadulterated union values */
+struct value_comparator
 {
   struct comparator parent;
-  double rounded_ref;
+  union value *pattern;
 };
 
 /* A comparator which matches string values or parts thereof */
@@ -455,53 +458,32 @@ struct regexp_comparator
   regex_t re;
 };
 
-/* Returns 10 raised to the power of X.
-   X must be a non-negative integer.  */
-static inline int
-int_pow10 (int x)
-{
-  int ret = 1;
-  assert (x >= 0);
-  while (x--)
-    ret *= 10;
-
-  return ret;
-}
 
 static bool
 value_compare (const struct comparator *cmptr,
-               const union value *v)
+	       const union value *v)
 {
-  const struct numeric_comparator *nc = (const struct numeric_comparator *) cmptr;
-  struct fmt_spec fs = var_get_print_format (cmptr->var);
-
-  double c = nearbyint (v->f * int_pow10 (fs.d));
-
-  return c == nc->rounded_ref;
+  const struct value_comparator *vc = (const struct value_comparator *) cmptr;
+  return 0 == compare_values (v, vc->pattern, var_get_width (cmptr->var));
 }
 
 
 /* Return true if the label of VAL matches the reference string*/
 static bool
 string_label_compare (const struct comparator *cmptr,
-                const union value *val)
+		const union value *val)
 {
   const struct string_comparator *ssc =
     (const struct string_comparator *) cmptr;
 
-  int width;
-
   const char *text = var_lookup_value_label (cmptr->var, val);
-  if (text == NULL)
-    return false;
+  int width = strlen (text);
 
-  width = strlen (text);
-
-  assert (cmptr->flags & STR_CMP_LABELS);
+  assert ( cmptr->flags & STR_CMP_LABELS);
 
   g_return_val_if_fail (width > 0, false);
 
-  if (cmptr->flags & STR_CMP_SUBSTR)
+  if ( cmptr->flags & STR_CMP_SUBSTR)
     return (NULL != g_strstr_len (text, width, ssc->pattern));
   else
     return (0 == strncmp (text, ssc->pattern, width));
@@ -510,26 +492,22 @@ string_label_compare (const struct comparator *cmptr,
 /* Return true if VAL matches the reference string*/
 static bool
 string_value_compare (const struct comparator *cmptr,
-                      const union value *val)
+		      const union value *val)
 {
-  bool found;
-  char *text;
   const struct string_comparator *ssc =
     (const struct string_comparator *) cmptr;
 
+  const char *text = val->s;
   int width = var_get_width (cmptr->var);
+
+  assert ( ! (cmptr->flags & STR_CMP_LABELS));
+
   g_return_val_if_fail (width > 0, false);
-  assert (! (cmptr->flags & STR_CMP_LABELS));
 
-  text = value_to_text (*val, cmptr->var);
-
-  if (cmptr->flags & STR_CMP_SUBSTR)
-    found =  (NULL != g_strstr_len (text, width, ssc->pattern));
+  if ( cmptr->flags & STR_CMP_SUBSTR)
+    return (NULL != g_strstr_len (text, width, ssc->pattern));
   else
-    found = (0 == strncmp (text, ssc->pattern, width));
-
-  free (text);
-  return found;
+    return (0 == strncmp (text, ssc->pattern, width));
 }
 
 
@@ -537,7 +515,7 @@ string_value_compare (const struct comparator *cmptr,
 /* Return true if VAL matched the regexp */
 static bool
 regexp_value_compare (const struct comparator *cmptr,
-                const union value *val)
+		const union value *val)
 {
   char *text;
   bool retval;
@@ -546,13 +524,13 @@ regexp_value_compare (const struct comparator *cmptr,
 
   int width = var_get_width (cmptr->var);
 
-  assert  (! (cmptr->flags & STR_CMP_LABELS));
+  assert  ( ! (cmptr->flags & STR_CMP_LABELS) );
 
   g_return_val_if_fail (width > 0, false);
 
-  text = value_to_text (*val, cmptr->var);
   /* We must remove trailing whitespace, otherwise $ will not match where
      one would expect */
+  text = g_strndup (val->s, width);
   g_strchomp (text);
 
   retval = (0 == regexec (&rec->re, text, 0, 0, 0));
@@ -565,7 +543,7 @@ regexp_value_compare (const struct comparator *cmptr,
 /* Return true if the label of VAL matched the regexp */
 static bool
 regexp_label_compare (const struct comparator *cmptr,
-                      const union value *val)
+		      const union value *val)
 {
   const char *text;
   const struct regexp_comparator *rec =
@@ -573,7 +551,7 @@ regexp_label_compare (const struct comparator *cmptr,
 
   int width ;
 
-  assert (cmptr->flags & STR_CMP_LABELS);
+  assert ( cmptr->flags & STR_CMP_LABELS);
 
   text = var_lookup_value_label (cmptr->var, val);
   width = strlen (text);
@@ -588,42 +566,60 @@ regexp_label_compare (const struct comparator *cmptr,
 static void
 regexp_destroy (struct comparator *cmptr)
 {
-  struct regexp_comparator *rec
-    = UP_CAST (cmptr, struct regexp_comparator, parent);
+  struct regexp_comparator *rec = (struct regexp_comparator *) cmptr;
 
   regfree (&rec->re);
 }
 
-static struct comparator *
-numeric_comparator_create (const struct variable *var, const char *target)
+static void
+value_destroy (struct comparator *cmptr)
 {
-  struct numeric_comparator *nc = XZALLOC (struct numeric_comparator);
-  struct comparator *cmptr = &nc->parent;
+  struct value_comparator *vc = (struct value_comparator *) cmptr;
+  free (vc->pattern);
+}
+
+
+static struct comparator *
+value_comparator_create (const struct variable *var, const char *target)
+{
+  const struct fmt_spec *fmt;
+  int width ;
+  struct value_comparator *vc = xzalloc (sizeof (*vc));
+  struct comparator *cmptr = (struct comparator *) vc;
 
   cmptr->flags = 0;
   cmptr->var = var;
-  cmptr->compare  = value_compare;
-  struct fmt_spec fs = var_get_write_format (var);
+  cmptr->compare  = value_compare ;
+  cmptr->destroy = value_destroy;
 
-  union value val;
-  text_to_value (target, var, &val);
-  nc->rounded_ref = nearbyint (val.f * int_pow10 (fs.d));
-  value_destroy (&val, var_get_width (var));
+  width = var_get_width (var);
+  fmt = var_get_write_format (var);
+
+  vc->pattern = value_create (width);
+
+  if ( ! data_in (ss_cstr (target),
+		  fmt->type,
+		  0, 0,
+		  vc->pattern, width) )
+    {
+      free (vc);
+      return NULL;
+    }
 
   return cmptr;
 }
 
 static struct comparator *
 string_comparator_create (const struct variable *var, const char *target,
-                          enum string_cmp_flags flags)
+			  enum string_cmp_flags flags)
 {
-  struct string_comparator *ssc = XZALLOC (struct string_comparator);
-  struct comparator *cmptr = &ssc->parent;
+  struct string_comparator *ssc = xzalloc (sizeof (*ssc));
+  struct comparator *cmptr = (struct comparator *) ssc;
 
   cmptr->flags = flags;
   cmptr->var = var;
 
-  if (flags & STR_CMP_LABELS)
+  if ( flags & STR_CMP_LABELS)
     cmptr->compare = string_label_compare;
   else
     cmptr->compare = string_value_compare;
@@ -636,11 +632,11 @@ string_comparator_create (const struct variable *var, const char *target,
 
 static struct comparator *
 regexp_comparator_create (const struct variable *var, const char *target,
-                          enum string_cmp_flags flags)
+			  enum string_cmp_flags flags)
 {
   int code;
-  struct regexp_comparator *rec = XZALLOC (struct regexp_comparator);
-  struct comparator *cmptr = &rec->parent;
+  struct regexp_comparator *rec = xzalloc (sizeof (*rec));
+  struct comparator *cmptr = (struct comparator *) rec;
 
   cmptr->flags = flags;
   cmptr->var = var;
@@ -650,7 +646,7 @@ regexp_comparator_create (const struct variable *var, const char *target,
   cmptr->destroy  = regexp_destroy;
 
   code = regcomp (&rec->re, target, 0);
-  if (code != 0)
+  if ( code != 0 )
     {
       char *errbuf = NULL;
       size_t errbuf_size = regerror (code, &rec->re, errbuf,  0);
@@ -661,7 +657,7 @@ regexp_comparator_create (const struct variable *var, const char *target,
 
       msg (ME, _("Bad regular expression: %s"), errbuf);
 
-      free (cmptr);
+      free ( cmptr);
       free (errbuf);
       return NULL;
     }
@@ -673,7 +669,7 @@ regexp_comparator_create (const struct variable *var, const char *target,
 /* Compare V against CMPTR's reference */
 static bool
 comparator_compare (const struct comparator *cmptr,
-                    const union value *v)
+		    const union value *v)
 {
   return cmptr->compare (cmptr, v);
 }
@@ -682,10 +678,10 @@ comparator_compare (const struct comparator *cmptr,
 static void
 comparator_destroy (struct comparator *cmptr)
 {
-  if (! cmptr)
+  if ( ! cmptr )
     return ;
 
-  if (cmptr->destroy)
+  if ( cmptr->destroy )
     cmptr->destroy (cmptr);
 
   free (cmptr);
@@ -694,15 +690,15 @@ comparator_destroy (struct comparator *cmptr)
 
 static struct comparator *
 comparator_factory (const struct variable *var, const char *str,
-                    enum string_cmp_flags flags)
+		    enum string_cmp_flags flags)
 {
-  if (flags & STR_CMP_REGEXP)
+  if ( flags & STR_CMP_REGEXP )
     return regexp_comparator_create (var, str, flags);
 
-  if (flags & (STR_CMP_SUBSTR | STR_CMP_LABELS))
+  if ( flags & (STR_CMP_SUBSTR | STR_CMP_LABELS) )
     return string_comparator_create (var, str, flags);
 
-  return numeric_comparator_create (var, str);
+  return value_comparator_create (var, str);
 }
 
 
@@ -712,7 +708,7 @@ comparator_factory (const struct variable *var, const char *str,
  */
 static void
 find_value (const struct find_dialog *fd, casenumber current_row,
-           casenumber *row, int *column)
+	   casenumber *row, int *column)
 {
   int width;
   const struct variable *var;
@@ -720,9 +716,10 @@ find_value (const struct find_dialog *fd, casenumber current_row,
   const char *target_string = gtk_entry_get_text (GTK_ENTRY (fd->value_entry));
 
   enum string_cmp_flags flags = 0;
+  g_assert (current_row >= 0);
 
   var = dict_lookup_var (fd->dict->dict, var_name);
-  if (! var)
+  if ( ! var )
     return ;
 
   width = var_get_width (var);
@@ -730,44 +727,44 @@ find_value (const struct find_dialog *fd, casenumber current_row,
   *column = var_get_dict_index (var);
   *row = -1;
 
-  if (gtk_toggle_button_get_active
+  if ( gtk_toggle_button_get_active
        (GTK_TOGGLE_BUTTON (fd->match_substring_checkbox)))
     flags |= STR_CMP_SUBSTR;
 
-  if (gtk_toggle_button_get_active
+  if ( gtk_toggle_button_get_active
        (GTK_TOGGLE_BUTTON (fd->match_regexp_checkbox)))
     flags |= STR_CMP_REGEXP;
 
-  if (gtk_toggle_button_get_active
+  if ( gtk_toggle_button_get_active
        (GTK_TOGGLE_BUTTON (fd->value_labels_checkbox)))
     flags |= STR_CMP_LABELS;
 
   {
-    union value val;
+    union value *val = value_create (width);
     casenumber i;
-    const struct casenum_iterator *ip = get_iteration_params (fd);
+    struct casenum_iterator *ip = get_iteration_params (fd);
     struct comparator *cmptr =
       comparator_factory (var, target_string, flags);
 
-    value_init (&val, width);
-    if (! cmptr)
+    if ( ! cmptr)
       goto finish;
 
     for (i = ip->start (current_row, fd->data);
-         i != ip->end (current_row, fd->data);
-         ip->next (&i, fd->data))
+	 i != ip->end (current_row, fd->data);
+	 ip->next (&i, fd->data))
       {
-        datasheet_get_value (fd->data, i, var_get_dict_index (var), &val);
+	datasheet_get_value (fd->data, i, var_get_case_index (var),
+			     val, width);
 
-        if (comparator_compare (cmptr, &val))
-          {
-            *row = i;
-            break;
-          }
+	if ( comparator_compare (cmptr, val))
+	  {
+	    *row = i;
+	    break;
+	  }
       }
 
   finish:
     comparator_destroy (cmptr);
-    value_destroy (&val, width);
+    free (val);
   }
 }
