@@ -1,11 +1,9 @@
-/* psppire-var-store.c
-
-   PSPPIRE --- A Graphical User Interface for PSPP
+/* PSPPIRE - a graphical user interface for PSPP.
    Copyright (C) 2006  Free Software Foundation
 
-   This program is free software; you can redistribute it and/or modify
+   This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
+   the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.
 
    This program is distributed in the hope that it will be useful,
@@ -14,9 +12,7 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
-   02110-1301, USA. */
+   along with this program.  If not, see <http://www.gnu.org/licenses/>. */
 
 #include <config.h>
 #include <string.h>
@@ -43,6 +39,7 @@
 #include "missing-val-dialog.h"
 #include <data/value-labels.h>
 
+#include "var-display.h"
 
 #define TRAILING_ROWS 40
 
@@ -51,15 +48,20 @@ static void         psppire_var_store_class_init      (PsppireVarStoreClass *cla
 static void         psppire_var_store_sheet_model_init (GSheetModelIface *iface);
 static void         psppire_var_store_finalize        (GObject           *object);
 
-static gchar *psppire_var_store_get_string (const GSheetModel *sheet_model, gint row, gint column);
 
-static gboolean  psppire_var_store_clear (GSheetModel *model,  gint row, gint col);
+gchar * missing_values_to_string (const struct variable *pv, GError **err);
+
+
+static gchar *psppire_var_store_get_string (const GSheetModel *sheet_model, glong row, glong column);
+
+static gboolean  psppire_var_store_clear (GSheetModel *model,  glong row, glong col);
 
 
 static gboolean psppire_var_store_set_string (GSheetModel *model,
-					  const gchar *text, gint row, gint column);
+					  const gchar *text, glong row, glong column);
 
-static gint psppire_var_store_get_row_count (const GSheetModel * model);
+static glong psppire_var_store_get_row_count (const GSheetModel * model);
+static glong psppire_var_store_get_column_count (const GSheetModel * model);
 
 static gchar *text_for_column (const struct variable *pv, gint c, GError **err);
 
@@ -145,7 +147,7 @@ psppire_var_store_init (PsppireVarStore *var_store)
 }
 
 static gboolean
-psppire_var_store_item_editable (PsppireVarStore *var_store, gint row, gint column)
+psppire_var_store_item_editable (PsppireVarStore *var_store, glong row, glong column)
 {
   const struct fmt_spec *write_spec ;
 
@@ -154,10 +156,10 @@ psppire_var_store_item_editable (PsppireVarStore *var_store, gint row, gint colu
   if ( !pv )
     return TRUE;
 
-  if ( VAR_STRING == var_get_type (pv) && column == COL_DECIMALS )
+  if ( var_is_alpha (pv) && column == COL_DECIMALS )
     return FALSE;
 
-  write_spec =var_get_write_format (pv);
+  write_spec = var_get_print_format (pv);
 
   switch ( write_spec->type )
     {
@@ -186,13 +188,13 @@ psppire_var_store_item_editable (PsppireVarStore *var_store, gint row, gint colu
 
 
 struct variable *
-psppire_var_store_get_var (PsppireVarStore *store, gint row)
+psppire_var_store_get_var (PsppireVarStore *store, glong row)
 {
   return psppire_dict_get_variable (store->dict, row);
 }
 
 static gboolean
-psppire_var_store_is_editable (const GSheetModel *model, gint row, gint column)
+psppire_var_store_is_editable (const GSheetModel *model, glong row, glong column)
 {
   PsppireVarStore *store = PSPPIRE_VAR_STORE (model);
   return psppire_var_store_item_editable (store, row, column);
@@ -200,7 +202,7 @@ psppire_var_store_is_editable (const GSheetModel *model, gint row, gint column)
 
 
 static const GdkColor *
-psppire_var_store_get_foreground (const GSheetModel *model, gint row, gint column)
+psppire_var_store_get_foreground (const GSheetModel *model, glong row, glong column)
 {
   PsppireVarStore *store = PSPPIRE_VAR_STORE (model);
 
@@ -213,7 +215,7 @@ psppire_var_store_get_foreground (const GSheetModel *model, gint row, gint colum
 
 const PangoFontDescription *
 psppire_var_store_get_font_desc (const GSheetModel *model,
-			      gint row, gint column)
+			      glong row, glong column)
 {
   PsppireVarStore *store = PSPPIRE_VAR_STORE (model);
 
@@ -222,10 +224,12 @@ psppire_var_store_get_font_desc (const GSheetModel *model,
 
 
 
+
 static void
 psppire_var_store_sheet_model_init (GSheetModelIface *iface)
 {
   iface->get_row_count = psppire_var_store_get_row_count;
+  iface->get_column_count = psppire_var_store_get_column_count;
   iface->free_strings = TRUE;
   iface->get_string = psppire_var_store_get_string;
   iface->set_string = psppire_var_store_set_string;
@@ -270,24 +274,30 @@ var_change_callback (GtkWidget *w, gint n, gpointer data)
 
 
 static void
-var_delete_callback (GtkWidget *w, gint first, gint n, gpointer data)
+var_delete_callback (GtkWidget *w, gint dict_idx, gint case_idx, gint val_cnt, gpointer data)
 {
   GSheetModel *model = G_SHEET_MODEL (data);
 
-  g_sheet_model_rows_deleted (model, first, n);
+  g_sheet_model_rows_deleted (model, dict_idx, 1);
 }
 
 
 
 static void
-var_insert_callback (GtkWidget *w, gint row, gpointer data)
+var_insert_callback (GtkWidget *w, glong row, gpointer data)
 {
   GSheetModel *model = G_SHEET_MODEL (data);
 
   g_sheet_model_rows_inserted (model, row, 1);
 }
 
+static void
+refresh (PsppireDict  *d, gpointer data)
+{
+  PsppireVarStore *vs = data;
 
+  g_sheet_model_range_changed (G_SHEET_MODEL (vs), -1, -1, -1, -1);
+}
 
 /**
  * psppire_var_store_replace_set_dictionary:
@@ -307,12 +317,14 @@ psppire_var_store_set_dictionary (PsppireVarStore *var_store, PsppireDict *dict)
   g_signal_connect (dict, "variable-changed", G_CALLBACK (var_change_callback),
 		   var_store);
 
-  g_signal_connect (dict, "variables-deleted", G_CALLBACK (var_delete_callback),
+  g_signal_connect (dict, "variable-deleted", G_CALLBACK (var_delete_callback),
 		   var_store);
 
-  g_signal_connect (dict, "variable-inserted", G_CALLBACK (var_insert_callback),
-		   var_store);
+  g_signal_connect (dict, "variable-inserted",
+		    G_CALLBACK (var_insert_callback), var_store);
 
+  g_signal_connect (dict, "backend-changed", G_CALLBACK (refresh),
+		    var_store);
 
   /* The entire model has changed */
   g_sheet_model_range_changed (G_SHEET_MODEL (var_store), -1, -1, -1, -1);
@@ -326,7 +338,7 @@ psppire_var_store_finalize (GObject *object)
 }
 
 static gchar *
-psppire_var_store_get_string (const GSheetModel *model, gint row, gint column)
+psppire_var_store_get_string (const GSheetModel *model, glong row, glong column)
 {
   PsppireVarStore *store = PSPPIRE_VAR_STORE (model);
 
@@ -346,7 +358,7 @@ psppire_var_store_get_string (const GSheetModel *model, gint row, gint column)
    Returns true if anything was updated, false otherwise.
 */
 static gboolean
-psppire_var_store_clear (GSheetModel *model,  gint row, gint col)
+psppire_var_store_clear (GSheetModel *model,  glong row, glong col)
 {
   struct variable *pv ;
 
@@ -377,7 +389,7 @@ psppire_var_store_clear (GSheetModel *model,  gint row, gint col)
 */
 static gboolean
 psppire_var_store_set_string (GSheetModel *model,
-			  const gchar *text, gint row, gint col)
+			  const gchar *text, glong row, glong col)
 {
   struct variable *pv ;
 
@@ -464,11 +476,11 @@ psppire_var_store_set_string (GSheetModel *model,
 }
 
 
+const static gchar none[] = N_("None");
+
 static  gchar *
 text_for_column (const struct variable *pv, gint c, GError **err)
 {
-  static gchar none[] = N_("None");
-
   static const gchar *const type_label[] =
     {
       N_("Numeric"),
@@ -580,63 +592,7 @@ text_for_column (const struct variable *pv, gint c, GError **err)
 
     case COL_MISSING:
       {
-	gchar *s;
-	const struct missing_values *miss = var_get_missing_values (pv);
-	if ( mv_is_empty (miss))
-	  return g_locale_to_utf8 (gettext (none), -1, 0, 0, err);
-	else
-	  {
-	    if ( ! mv_has_range (miss))
-	      {
-		GString *gstr = g_string_sized_new (10);
-		const int n = mv_n_values (miss);
-		gchar *mv[4] = {0,0,0,0};
-		gint i;
-		for (i = 0 ; i < n; ++i )
-		  {
-		    union value v;
-		    mv_peek_value (miss, &v, i);
-		    mv[i] = value_to_text (v, *write_spec);
-		    if ( i > 0 )
-		      g_string_append (gstr, ", ");
-		    g_string_append (gstr, mv[i]);
-		    g_free (mv[i]);
-		  }
-		s = pspp_locale_to_utf8 (gstr->str, gstr->len, err);
-		g_string_free (gstr, TRUE);
-	      }
-	    else
-	      {
-		GString *gstr = g_string_sized_new (10);
-		gchar *l, *h;
-		union value low, high;
-		mv_peek_range (miss, &low.f, &high.f);
-
-		l = value_to_text (low, *write_spec);
-		h = value_to_text (high, *write_spec);
-
-		g_string_printf (gstr, "%s - %s", l, h);
-		g_free (l);
-		g_free (h);
-
-		if ( mv_has_value (miss))
-		  {
-		    gchar *ss = 0;
-		    union value v;
-		    mv_peek_value (miss, &v, 0);
-
-		    ss = value_to_text (v, *write_spec);
-
-		    g_string_append (gstr, ", ");
-		    g_string_append (gstr, ss);
-		    free (ss);
-		  }
-		s = pspp_locale_to_utf8 (gstr->str, gstr->len, err);
-		g_string_free (gstr, TRUE);
-	      }
-
-	    return s;
-	  }
+	return missing_values_to_string (pv, err);
       }
       break;
     case COL_VALUES:
@@ -678,11 +634,7 @@ text_for_column (const struct variable *pv, gint c, GError **err)
       break;
     case COL_MEASURE:
       {
-	const gint measure = var_get_measure (pv);
-
-	g_assert (measure < n_MEASURES);
-	return g_locale_to_utf8 (gettext (measures[measure]),
-				 -1, 0, 0, err);
+	return measure_to_string (pv, err);
       }
       break;
     }
@@ -711,7 +663,7 @@ psppire_var_store_set_font (PsppireVarStore *store, const PangoFontDescription *
 }
 
 
-static gint
+static glong
 psppire_var_store_get_row_count (const GSheetModel * model)
 {
   gint rows = 0;
@@ -723,9 +675,16 @@ psppire_var_store_get_row_count (const GSheetModel * model)
   return rows ;
 }
 
+static glong
+psppire_var_store_get_column_count (const GSheetModel * model)
+{
+  return n_COLS ;
+}
+
+
 /* Row related funcs */
 
-static gint
+static glong
 geometry_get_row_count (const GSheetRow *geom, gpointer data)
 {
   gint rows = 0;
@@ -739,14 +698,14 @@ geometry_get_row_count (const GSheetRow *geom, gpointer data)
 
 
 static gint
-geometry_get_height (const GSheetRow *geom, gint row, gpointer data)
+geometry_get_height (const GSheetRow *geom, glong row, gpointer data)
 {
   return 25;
 }
 
 
 static gboolean
-geometry_is_sensitive (const GSheetRow *geom, gint row, gpointer data)
+geometry_is_sensitive (const GSheetRow *geom, glong row, gpointer data)
 {
   PsppireVarStore *vs = PSPPIRE_VAR_STORE (geom);
 
@@ -764,14 +723,13 @@ gboolean always_true ()
 
 
 static gchar *
-geometry_get_button_label (const GSheetRow *geom, gint unit, gpointer data)
+geometry_get_button_label (const GSheetRow *geom, glong unit, gpointer data)
 {
-  gchar *label = g_strdup_printf (_("%d"), unit);
+  gchar *label = g_strdup_printf (_("%ld"), unit);
 
   return label;
 }
 
-
 static void
 psppire_var_store_sheet_row_init (GSheetRowIface *iface)
 {
@@ -783,3 +741,6 @@ psppire_var_store_sheet_row_init (GSheetRowIface *iface)
 
   iface->get_button_label = geometry_get_button_label;
 }
+
+
+
