@@ -1,5 +1,5 @@
 /* PSPPIRE - a graphical user interface for PSPP.
-   Copyright (C) 2007, 2010, 2011  Free Software Foundation
+   Copyright (C) 2007  Free Software Foundation
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -19,9 +19,12 @@
 
 #include <glib.h>
 #include <gtk/gtk.h>
+#include <gtk/gtksignal.h>
 #include "psppire-vbuttonbox.h"
 #include "psppire-dialog.h"
 
+#include <gtk/gtkbbox.h>
+
 #include <gettext.h>
 
 #define _(msgid) gettext (msgid)
@@ -37,31 +40,7 @@ static void gtk_vbutton_box_size_allocate (GtkWidget      *widget,
 					   GtkAllocation  *allocation);
 
 
-static void
-psppire_vbutton_box_get_preferred_height (GtkWidget *widget,
-                                gint      *minimal_height,
-                                gint      *natural_height)
-{
-  GtkRequisition requisition;
-
-  gtk_vbutton_box_size_request (widget, &requisition);
-
-  *minimal_height = *natural_height = requisition.height;
-}
-
-
-static void
-psppire_vbutton_box_get_preferred_width (GtkWidget *widget,
-                                gint      *minimal_width,
-                                gint      *natural_width)
-{
-  GtkRequisition requisition;
-
-  gtk_vbutton_box_size_request (widget, &requisition);
-
-  *minimal_width = *natural_width = requisition.width;
-}
-
+static GtkButtonBoxStyle default_layout_style = GTK_BUTTONBOX_EDGE;
 
 GType
 psppire_vbutton_box_get_type (void)
@@ -97,8 +76,7 @@ psppire_vbuttonbox_class_init (PsppireVButtonBoxClass *class)
 
   widget_class = (GtkWidgetClass*) class;
 
-  widget_class->get_preferred_width = psppire_vbutton_box_get_preferred_width;
-  widget_class->get_preferred_height = psppire_vbutton_box_get_preferred_height;
+  widget_class->size_request = gtk_vbutton_box_size_request;
   widget_class->size_allocate = gtk_vbutton_box_size_allocate;
 }
 
@@ -139,8 +117,9 @@ gtk_vbutton_box_size_request (GtkWidget      *widget,
   box = GTK_BOX (widget);
   bbox = GTK_BUTTON_BOX (widget);
 
-  spacing = gtk_box_get_spacing (box);
-  layout = gtk_button_box_get_layout (bbox);
+  spacing = box->spacing;
+  layout = bbox->layout_style != GTK_BUTTONBOX_DEFAULT_STYLE
+	  ? bbox->layout_style : default_layout_style;
 
   _psppire_button_box_child_requisition (widget,
 					 &nvis_children,
@@ -164,16 +143,18 @@ gtk_vbutton_box_size_request (GtkWidget      *widget,
       case GTK_BUTTONBOX_EDGE:
       case GTK_BUTTONBOX_START:
       case GTK_BUTTONBOX_END:
-      default:
         requisition->height =
 		nvis_children*child_height + ((nvis_children-1)*spacing);
 	break;
+      default:
+	    g_assert_not_reached();
+	    break;
       }
       requisition->width = child_width;
     }
 
-  requisition->width += gtk_container_get_border_width (GTK_CONTAINER (box)) * 2;
-  requisition->height += gtk_container_get_border_width (GTK_CONTAINER (box)) * 2;
+  requisition->width += GTK_CONTAINER (box)->border_width * 2;
+  requisition->height += GTK_CONTAINER (box)->border_width * 2;
 }
 
 
@@ -184,6 +165,7 @@ gtk_vbutton_box_size_allocate (GtkWidget     *widget,
 {
   GtkBox *base_box;
   GtkButtonBox *box;
+  GtkBoxChild *child;
   GList *children;
   GtkAllocation child_allocation;
   gint nvis_children;
@@ -201,28 +183,28 @@ gtk_vbutton_box_size_allocate (GtkWidget     *widget,
 
   base_box = GTK_BOX (widget);
   box = GTK_BUTTON_BOX (widget);
-  spacing = gtk_box_get_spacing (base_box);
-  layout = gtk_button_box_get_layout (box) ;
+  spacing = base_box->spacing;
+  layout = box->layout_style != GTK_BUTTONBOX_DEFAULT_STYLE
+	  ? box->layout_style : default_layout_style;
   _psppire_button_box_child_requisition (widget,
                                      &nvis_children,
 				     &n_secondaries,
                                      &child_width,
                                      &child_height);
-  gtk_widget_set_allocation (widget, allocation);
-  height = allocation->height - gtk_container_get_border_width (GTK_CONTAINER (box))*2;
+  widget->allocation = *allocation;
+  height = allocation->height - GTK_CONTAINER (box)->border_width*2;
   switch (layout)
   {
   case GTK_BUTTONBOX_SPREAD:
     childspacing = (height - (nvis_children * child_height)) / (nvis_children + 1);
-    y = allocation->y + gtk_container_get_border_width (GTK_CONTAINER (box)) + childspacing;
+    y = allocation->y + GTK_CONTAINER (box)->border_width + childspacing;
     secondary_y = y + ((nvis_children - n_secondaries) * (child_height + childspacing));
     break;
   case GTK_BUTTONBOX_EDGE:
-  default:
     if (nvis_children >= 2)
       {
         childspacing = (height - (nvis_children*child_height)) / (nvis_children-1);
-	y = allocation->y + gtk_container_get_border_width (GTK_CONTAINER (box));
+	y = allocation->y + GTK_CONTAINER (box)->border_width;
 	secondary_y = y + ((nvis_children - n_secondaries) * (child_height + childspacing));
       }
     else
@@ -234,41 +216,42 @@ gtk_vbutton_box_size_allocate (GtkWidget     *widget,
     break;
   case GTK_BUTTONBOX_START:
     childspacing = spacing;
-    y = allocation->y + gtk_container_get_border_width (GTK_CONTAINER (box));
+    y = allocation->y + GTK_CONTAINER (box)->border_width;
     secondary_y = allocation->y + allocation->height
       - child_height * n_secondaries
       - spacing * (n_secondaries - 1)
-      - gtk_container_get_border_width (GTK_CONTAINER (box));
+      - GTK_CONTAINER (box)->border_width;
     break;
   case GTK_BUTTONBOX_END:
     childspacing = spacing;
     y = allocation->y + allocation->height
       - child_height * (nvis_children - n_secondaries)
       - spacing * (nvis_children - n_secondaries - 1)
-      - gtk_container_get_border_width (GTK_CONTAINER (box));
-    secondary_y = allocation->y + gtk_container_get_border_width (GTK_CONTAINER (box));
+      - GTK_CONTAINER (box)->border_width;
+    secondary_y = allocation->y + GTK_CONTAINER (box)->border_width;
+    break;
+  default:
+    g_assert_not_reached();
     break;
   }
 
   x = allocation->x + (allocation->width - child_width) / 2;
   childspace = child_height + childspacing;
 
-  children = gtk_container_get_children (GTK_CONTAINER (box));
+  children = GTK_BOX (box)->children;
+
   while (children)
     {
-      GtkWidget *child = children->data;
+      child = children->data;
       children = children->next;
 
-      if (gtk_widget_get_visible (child))
+      if (GTK_WIDGET_VISIBLE (child->widget))
 	{
-          gboolean is_secondary = FALSE;
-          gtk_container_child_get (GTK_CONTAINER (box), child, "secondary", &is_secondary, NULL);
-
 	  child_allocation.width = child_width;
 	  child_allocation.height = child_height;
 	  child_allocation.x = x;
 
-	  if (is_secondary)
+	  if (child->is_secondary)
 	    {
 	      child_allocation.y = secondary_y;
 	      secondary_y += childspace;
@@ -279,7 +262,7 @@ gtk_vbutton_box_size_allocate (GtkWidget     *widget,
 	      y += childspace;
 	    }
 
-	  gtk_widget_size_allocate (child, &child_allocation);
+	  gtk_widget_size_allocate (child->widget, &child_allocation);
 	}
     }
 }
